#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/timeb.h>

#include "midi_b9.h"

static double Xcenter, Ycenter, Zcenter, posvink, j_RA, j_dec;
static double phas, cosvink, sinvink, cosph2, sinph2, cosj_dec, sinj_dec, cosj_RA, sinj_RA;

/* ******************************************************** */

int getRAdec_init(int jemx, double p_RA[10000], double p_dec[10000], double p_posang[10000], 
        struct euclid *g_const, struct log_con *logger, struct instr_data *jmx_id, struct backpro *backproj)
/*   Scan through the pointing data to find the pointing direction corresponding to aux_orbit/aux_pid */
{
   int i, n, p_orbit, p_pid, p_type, p_year, p_month, p_day, p_hour, p_min, p_sec, pointval;
   char dummy[100], pointfil[200];
   float p_durat;
   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "getRAdec_init");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   

   FILE *pofil;

      pointval = 0;
      sprintf(pointfil, "%s/pointing.dat", logger->sweethome2);
      pofil = fopen(pointfil, "rt");
      if (pofil == NULL) {
         sprintf(logger->tmptxt, "No pointing file found! (%s)\n", pointfil);
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      } else {
         for (i=0; i<1000000; i++) {
            fgets(dummy, 100, pofil);
            if (feof(pofil)) goto p_end;
            sscanf(dummy, "%d %d %d %lf %lf %lf %d %d %d %d %d %d %f",
               &p_orbit, &p_pid, &p_type, &p_RA[backproj->aux_pid], 
               &p_dec[backproj->aux_pid], &p_posang[backproj->aux_pid], 
               &p_year, &p_month, &p_day, &p_hour, &p_min, &p_sec, &p_durat);
            if ((p_orbit == backproj->aux_orbit) && (p_pid == backproj->aux_pid) && ((p_type % 2) == 0)) {
               pointval = 1;
               goto p_end;
            }
         }
      }
p_end:
      fclose(pofil);
   
      if (pointval > 0) {
/*       sprintf(logger->tmptxt, "Pointing: RA: %8.4lf,  dec: %8.4lf,  posang: %8.4lf\n", p_RA, p_dec, p_posang);
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
      	 j_RA = p_RA[backproj->aux_pid] * g_const->gtorad;
         j_dec = p_dec[backproj->aux_pid] * g_const->gtorad;
      
         Xcenter = cos(j_dec) * cos(j_RA);		/* X towards spring equinox */
         Ycenter = cos(j_dec) * sin(j_RA);		/* Y perpendicular to X and Z */
         Zcenter = sin(j_dec);			/* Z towards North */

      } else {
         sprintf(logger->tmptxt, "No matching pointing data found for orbit %3d and pid: %3d\n", 
            	backproj->aux_orbit, backproj->aux_pid);
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }
      
      posvink = (180.0 - p_posang[backproj->aux_pid]) * g_const->gtorad;

      cosvink = cos(posvink);
      sinvink = sin(posvink);
      cosph2 = cos(jmx_id->phase2);
      sinph2 = sin(jmx_id->phase2);
      cosj_dec = cos(j_dec);
      sinj_dec = sin(j_dec);
      cosj_RA = cos(j_RA);
      sinj_RA = sin(j_RA);
      
      sprintf(logger->tmptxt, "Pointing transformation initialized\n");
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
      
      /* function exit timing start */
      TT_now = clock();
      logger->func_times[func_local] += difftime(TT_now, logger->TT);
      logger->TT = TT_now;
      logger->func_calling_num = local_calling_num;
      /* function exit timing end */
   
      return(pointval);
}

/* *********************************************************** */

int getRAdec(float x, float y, int jemx, float *s_RA, float *s_dec, struct euclid *g_const, 
   struct log_con *logger, struct instr_data *jmx_id, struct backpro *backproj)
{
   
   double srcQ, srcR, Xcorr, Ycorr, SRCY, SRCZ, SRCX, SRC_R, Xsrc, Ysrc, Zsrc, Rsrc;
   float avX, avY, avr;
   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "getRAdec");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
               
   avX = x - (float)(backproj->FOV_radius);
   avY = y - (float)(backproj->FOV_radius);
   avX = avX + jmx_id->x2x * avX * avX;
   avY = avY + jmx_id->y2y * avY * avY;
      
   avr = sqrt((double)(avX*avX + avY*avY));

   Xcorr = (avX + jmx_id->offX) * cosph2 - (avY + jmx_id->offY) * sinph2;
   Ycorr = (avY + jmx_id->offY) * cosph2 + (avX + jmx_id->offX) * sinph2;

   Xcorr = -Xcorr;
   Ycorr = -Ycorr;

   srcQ = (Xcorr * jmx_id->scaleX * cosvink + Ycorr * jmx_id->scaleY * sinvink) * backproj->pix2rad + 
      	jmx_id->offQ; 
   srcR = (-Xcorr * jmx_id->scaleX * sinvink + Ycorr * jmx_id->scaleY * cosvink) * backproj->pix2rad + 
      	jmx_id->offR;	
               /* the Q-axis is in the mask plane, parallel to equator, srcQ is in radians */
               /* the R-coordinate is in mask plane, perpendicular to Q, srcR is in radians */

   Zsrc = srcR * cosj_dec;
   Xsrc = -srcQ * sinj_RA - srcR * sinj_dec * cosj_RA;
   Ysrc =  srcQ * cosj_RA - srcR * sinj_dec * sinj_RA;
   
   SRCX = Xcenter + Xsrc;
   SRCY = Ycenter + Ysrc;
   SRCZ = Zcenter + Zsrc;

   SRC_R = sqrt(SRCX*SRCX + SRCY*SRCY + SRCZ*SRCZ);       
                 
   *s_dec = asin(SRCZ / SRC_R);
   *s_RA = atan2(SRCY, SRCX);
   
   if (*s_RA < 0.0) *s_RA += g_const->twopi;                

   *s_dec /= g_const->gtorad;
   *s_RA /= g_const->gtorad; 
   
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return(0);
}

/* *********************************************************** */

int getxy(float s_RA, float s_dec, int jemx, float *x, float *y, struct euclid *g_const, 
   	struct instr_data *jmx_id, struct backpro *backproj, struct log_con *logger)
{
   
   double srcQ, srcR, Xcorr, Ycorr, SRCY, SRCZ, SRCX, SRC_R, Xsrc, Ysrc, Zsrc, Rsrc;
   double dist2, dist, ksi, cosksi, rr2;
   float avX, avY, avr;
   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "getxy");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
               
   s_dec *= g_const->gtorad;
   s_RA *= g_const->gtorad; 
   
   SRCX = cos((double)(s_dec)) * cos((double)(s_RA));
   SRCY = cos((double)(s_dec)) * sin((double)(s_RA));
   SRCZ = sin((double)(s_dec));   

   dist2 = (SRCX-Xcenter)*(SRCX-Xcenter) + (SRCY-Ycenter)*(SRCY-Ycenter) + (SRCZ-Zcenter)*(SRCZ-Zcenter);
   dist = sqrt(dist2);
   
   ksi = atan(dist / sqrt(4.0 - dist2));
   
   cosksi = cos(2.0 * ksi);
   SRCX /= cosksi;
   SRCY /= cosksi;
   SRCZ /= cosksi;
   
   Xsrc = SRCX  - Xcenter;
   Ysrc = SRCY  - Ycenter;
   Zsrc = SRCZ  - Zcenter;
   
   srcR = Zsrc / cosj_dec;
   rr2 = Xsrc*Xsrc + Ysrc*Ysrc + Zsrc*Zsrc;
   srcQ = sqrt(rr2 - srcR * srcR);
   if (j_RA > s_RA) srcQ = -srcQ;
   
   Xcorr = ((srcQ - jmx_id->offQ) * cosvink - (srcR - jmx_id->offR) * sinvink) / backproj->pix2rad;
   Ycorr = ((srcQ - jmx_id->offQ) * sinvink + (srcR - jmx_id->offR) * cosvink) / backproj->pix2rad;

   if (jemx == 1) {
      Xcorr = -Xcorr;
      Ycorr = -Ycorr;
   }
            
   avX = (Xcorr * cosph2 + Ycorr * sinph2) - jmx_id->offX;
   avY = (-Xcorr * sinph2 + Ycorr * cosph2) - jmx_id->offY;
   
   *x = avX - jmx_id->x2x * avX * avX;
   *y = avY - jmx_id->y2y * avY * avY;
   
   *x += (float)(backproj->FOV_radius);
   *y += (float)(backproj->FOV_radius);

   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return(0);

}

