#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/timeb.h>

#include "midi_b9.h"

int irosloop(int jemx, int ee, float xx[sourcmax], float yy[sourcmax], float norm_d[sourcmax], 
   	double solut[sourcmax], float sumfun[sourcmax], float *goodrms, double *sh_rms2, int *goodfits, 
        int *goodnsrc, float resid[detdim], float edge, float goodbet_g[sourcmax], int iter, 
        double limit_fraction, struct euclid *g_const, struct instr_data *jmx_id,
        struct log_con *logger, struct backpro *backproj)
{
      int i, j, k, m, sky_index, blindnum, tours, updates[sourcmax], nsrc, ngrp, fitcount;
      int mini_rms, mini_const, sxy, sxy_list[sourcmax], blindspots[sourcmax], just_updated;
      int smx[max_candidates], smy[max_candidates], status;
      float valmin_rms, valmin_const, xnow, ynow, varim[skydim], skyim[skydim], shadow_counts, funval;
      double amx[max_candidates+1], rms;
      char filnavn[100];

   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "irosloop");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
      blindnum = 0;
      tours = 0;
      for (i=0; i<sourcmax; i++) updates[i] = 0; 

      nsrc = backproj->backmodels;
      ngrp = nsrc - 1;

      if (logger->test == 1) {
         sprintf(logger->tmptxt, "Calling fun initially\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }
      status = fun(-1, 1, ee, xx, yy, jemx, norm_d, 0, &funval, g_const, jmx_id, logger, backproj);
         		/* reset fitting functions */
      if (status < 0) return(-1);
      
      if (logger->test == 1) {
         sprintf(logger->tmptxt, "Returning from fun\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }

      shadow_counts = 0;
      for (i=0; i<detdim; i++) if (backproj->soft_gr[i] > 0.0) shadow_counts += backproj->shadw[i];

      /* ***************************************************************** */
      /*                                                                   */
      /*      Backproject raw shadowgram to form variance image            */
      /*                                                                   */
      /* ***************************************************************** */
      
      if (logger->test == 1) {
         sprintf(logger->tmptxt, "Calling backpro_d\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }
      
      status = backpro_d(backproj->shadw, varim, logger, backproj);
      if (status < 0) return(-1);

      if (logger->test == 1) {
         sprintf(logger->tmptxt, "Returning from backpro_d - yes!\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }

      if (logger->test == 1) {
         sprintf(logger->tmptxt, "start i-loop: %5d", i); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }
      for (i=0; i<skydim; i++) {
         backproj->fitsimage[ee][0][i] = varim[i] * 1.0555; 
      }
         	/* 1.0555 = 1 + 1/18 (background contribution to variance) */

      if (logger->test == 1) {
         sprintf(logger->tmptxt, "Passed i-loop  - yes!\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }

      /* ******************************************************** */
      /*                                                          */
      /*       Fit shadowgram with background components          */
      /*                                                          */
      /* ******************************************************** */
      
	/* imagefit returns a solution to the fit process in "solut" and a residual shadowgram in "resid" */

      if (logger->test == 1) {
         sprintf(logger->tmptxt, "Calling imagefit\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }
      status = imagefit(backproj->shadw, jemx, ee, resid, nsrc, xx, yy, solut, norm_d, sumfun, 0, 
         	&rms, g_const, jmx_id, logger, backproj);
      if (status < 0) {
         printf("Found negative status on return from imagefit (1), status: %d\n", status); 
         goto nextee;
      }
      
      if (logger->test == 1) {
         sprintf(logger->tmptxt, "Returning from imagefit\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }
      fitcount = 1;

      if (rms < 0.0) return(-1);	/* bad data */

      *sh_rms2 = rmscalc(resid, backproj->soft_gr, logger);
      if (logger->test == 1) {
         sprintf(logger->tmptxt, "Returning from rmscalc\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }

      mini_rms = mini_const = 0;
      valmin_const = fabs(solut[0]);
      valmin_rms = rms;
      
      /* ******************************************************** */
      /*                                                          */
      /*    Backproject residual shadowgram to form sky image     */
      /*                                                          */
      /* ******************************************************** */
      
      if (logger->test == 1) {
         sprintf(logger->tmptxt, "Calling backpro_d 2\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }
      
      status = backpro_d(resid, backproj->skyim_r, logger, backproj);
      if (status < 0) return(-1);
      
      if (logger->test == 1) {
         sprintf(logger->tmptxt, "Returning from backpro_d 2\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }
      
      for (i=0; i<skydim; i++) 
         backproj->fitsimage[ee][1][i] = backproj->fitsimage[ee][4][i] = backproj->skyim_r[i]; 
    
      if (logger->test == 1) {
         sprintf(logger->tmptxt, "Calling clean\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }
      clean(backproj->skyim_r, skyim, logger);		/* remove large scale structures in skyimage */
      if (logger->test == 1) {
         sprintf(logger->tmptxt, "Returning from clean\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }
         
      for (i=0; i<skydim; i++) backproj->fitsimage[ee][2][i] = skyim[i]; 

	/* after first fit output background cleaned map */ 
igen:
      /* ******************************************************** */
      /* ******************************************************** */
      /* *                                                      * */
      /* *      Iterative source removal loop starts here       * */
      /* *                                                      * */
      /* ******************************************************** */
      /* ******************************************************** */

      nsrc++;
      tours++;
      if (nsrc >= 19) {
/*         sprintf(logger->tmptxt, " ############ stop due to nsrc = 19\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
   	 putchar((char)('0'+1));
         goto nextee;	/* next energy range */
      } 
               
      /* ******************************************************** */
      /*                                                          */
      /*            Find dominant source in sky image             */
      /*                                                          */
      /* ******************************************************** */

      if (blindnum > 0) {
         for (i=0; i<blindnum; i++) {
            j= blindspots[i];
            skyim[j] = 0.0;
            skyim[j+sky_ydim+1] = 0.0;
            skyim[j-sky_ydim-1] = 0.0;
            skyim[j+sky_ydim+2] = 0.0;
            skyim[j-sky_ydim-2] = 0.0;
            skyim[j+sky_ydim] = 0.0;
            skyim[j-sky_ydim] = 0.0;
            skyim[j + 1] = 0.0;
            skyim[j - 1] = 0.0;
            skyim[j+2*sky_ydim+2] = 0.0;
            skyim[j-2*sky_ydim-2] = 0.0;
            skyim[j+2*sky_ydim+3] = 0.0;
            skyim[j-2*sky_ydim-3] = 0.0;
            skyim[j+2*sky_ydim+4] = 0.0;
            skyim[j-2*sky_ydim-4] = 0.0;
            skyim[j + 2] = 0.0;
            skyim[j - 2] = 0.0;
            skyim[j+2*sky_ydim+1] = 0.0;
            skyim[j-2*sky_ydim-1] = 0.0;
            skyim[j+2*sky_ydim] = 0.0;
            skyim[j-2*sky_ydim] = 0.0;
            skyim[j+sky_ydim+3] = 0.0;
            skyim[j-sky_ydim-3] = 0.0;
            skyim[j+sky_ydim-1] = 0.0;
            skyim[j-sky_ydim+1] = 0.0;
         }
      }

	/* findpeak finds the highest peak in the sky image */

      sxy = findpeak(skyim, smx, smy, amx, edge, logger, backproj);
      
      if (sxy < 0) {
         putchar((char)('0'+7)); 
/*       sprintf(logger->tmptxt, " ############ stop for bad resiual map \n");  
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
   	 if (iter > 2) {
            ngrp--; 
            nsrc--; 	/* disregard last found source */
         }
         goto nextee;
      }
      
      if ((nsrc-backproj->backmodels) >= iter) {
  
         putchar((char)('0'+2)); 

/*       sprintf(logger->tmptxt, " ############ stop for iter\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
   	 if (iter > 2) {
            ngrp--; 
            nsrc--; 	/* disregard last found source */
         }
         goto nextee;
      }

/*      tours = 20;
*/
      if (tours >= 20) {
         putchar((char)('0'+3)); 
      
/*       sprintf(logger->tmptxt, " ############ stop for more than 20 fit cycles\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/  
   	 ngrp--; 
         nsrc--; 	/* disregard last found source */
         goto nextee;
      }
      if (amx[0] < (3.0 * amx[max_candidates])) {

/*       sprintf(logger->tmptxt, " ############ stop for peak significance\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
   	 ngrp--; 
         nsrc--; 	/* disregard last found source */
         putchar((char)('0'+4));
         goto nextee;	/* check significance of new peak */
      }
      
      xnow = amx[1] * backproj->depth_corr;
      ynow = amx[2] * backproj->depth_corr;

      ngrp = nsrc - 1;
      xx[ngrp] = xnow;
      yy[ngrp] = ynow;
      sxy_list[ngrp] = sxy;
      updates[ngrp] = 0;
      
/*      sprintf(logger->tmptxt, 
         "Dominant source position:  sxy: %5d,  XY: %+4d %+4d, delta xy: %+4d, %+2d, ngrp: %2d, fits: %d\n",
         sxy, (int)(sxy/sky_ydim - backproj->FOV_radius), (int)((sxy%sky_ydim) - backproj->FOV_radius), 
         smx[1], smy[1], ngrp, fitcount[ee]);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
      just_updated = 0;

      /* ******************************************************** */
      /*                                                          */
      /*   Fit shadowgram with background and source components   */
      /* Generate residual shadowgram for next findpeak operation */
      /*                                                          */
      /* ******************************************************** */
newfit:      
      status = imagefit(backproj->shadw, jemx, ee, resid, nsrc, xx, yy, solut, norm_d, sumfun, 0, 
         	&rms, g_const, jmx_id, logger, backproj);
      if (status < 0) {
/*         sprintf(logger->tmptxt, " ############ stop for negative status"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
         putchar((char)('0'+9));
         goto nextee;
      }
      
/*      if (status < 0) {
         printf("Found negative status on return from imagefit (2), status: %d\n", status); 
         return(-1);
      }
*/      
      fitcount++;
/*         sprintf(logger->tmptxt, 
   		"\n peak peak power: %8.3lf, counts: %10.1f\n", solut[ngrp], shadow_counts); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/      
      
      if (rms < 0.0) {
/*         sprintf(logger->tmptxt, " ############ stop for negative rms"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
         putchar((char)('0'+5));
         goto nextee;
      }
      
      if (just_updated == 1) goto no_update_check;
      
      	/* check for previously found peak */ 
      if (ngrp-backproj->backmodels > 0) {
         for (m=backproj->backmodels; m<ngrp; m++) {	
            if (((xnow-xx[m])*(xnow-xx[m])+(ynow-yy[m])*(ynow-yy[m])) < 6.0) {
               if (solut[ngrp] >= 0.0) {
                  if (solut[m] >= 0.0) {
                     xx[m] = (xx[m] * solut[m] + xnow * solut[ngrp]) / (solut[m] + solut[ngrp]);
                     yy[m] = (yy[m] * solut[m] + ynow * solut[ngrp]) / (solut[m] + solut[ngrp]);
                  } else {
                     xx[m] = xnow + (xnow - xx[m]) * solut[m] / (fabs(solut[m]) + solut[ngrp]);
                     yy[m] = ynow + (ynow - yy[m]) * solut[m] / (fabs(solut[m]) + solut[ngrp]);
                  }
               } else {
                  if (solut[m] >= 0.0) {
                     xx[m] = xx[m] + (xnow - xx[m]) * solut[ngrp] / (solut[m] - fabs(solut[ngrp]));
                     yy[m] = yy[m] + (ynow - yy[m]) * solut[ngrp] / (solut[m] - fabs(solut[ngrp]));
                  } else {
                     xx[m] = (xx[m] * solut[m] + xnow * solut[ngrp]) / (solut[m] + solut[ngrp]);
                     yy[m] = (yy[m] * solut[m] + ynow * solut[ngrp]) / (solut[m] + solut[ngrp]);
                  }
               }
               sxy = sxy_list[m];
               ngrp--;
               nsrc--;
               updates[m]++;
               just_updated = 1;
               if (updates[m] >= 5) {
                  blindspots[blindnum] = sxy;
                  blindnum++;
               }
/*               sprintf(logger->tmptxt, 
                  "%d update for source %1d position:  sxy: %5d,  XY: %6.1f %6.1f, ngrp: %2d, blind: %d\n",
                  updates[m], m, sxy, xx[m], yy[m], ngrp, blindnum);
               logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
               goto newfit;
            } 
         }
         for (m=backproj->backmodels; m<=ngrp; m++) updates[m] = 0;

         if (ngrp > backproj->backmodels) {
            if (rms < valmin_rms) valmin_rms = rms;
            else mini_rms++;
            if (fabs(solut[0]) < valmin_const) valmin_const = fabs(solut[0]);
            else mini_const++;
         } else {
            if (rms < valmin_rms) valmin_rms = rms;
            if (fabs(solut[0]) < valmin_const) valmin_const = fabs(solut[0]);
         }

no_update_check:
/*         sprintf(logger->tmptxt, 
   		"Update goodbet (if not past minimum: rms# %1d const: %1d): \n", mini_rms, mini_const);
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
         if ((mini_rms == 0) || (mini_const == 0)) {
            for (m=0; m<=ngrp; m++) goodbet_g[m] = solut[m];
            *goodnsrc = nsrc;
            *goodrms = rms;
            *goodfits = fitcount;
         } else {
/*            sprintf(logger->tmptxt, " ############ stop beyond best fit\n"); 
            logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
   	    putchar((char)('0'+6));
            goto nextee;
         }  
         if (solut[ngrp] < limit_fraction * shadow_counts) {
/*     	    sprintf(logger->tmptxt, 
   		"\n ############ stop for peak power:  solut[ngrp]: %8.3lf, counts: %10.1f\n",
               solut[ngrp], shadow_counts); 
            logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
   	    putchar((char)('0'+8));
            goto nextee;
         }
      
      }

      /* ******************************************************** */
      /*                                                          */
      /*    Backproject residual shadowgram to form sky image     */
      /*                                                          */
      /* ******************************************************** */

just_eliminate:      
      if (logger->test == 1) {
         sprintf(logger->tmptxt, "Calling backpro_d 3\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }
      
      backpro_d(resid, backproj->skyim_r, logger, backproj);
      if (status < 0) return(-1);
      
      if (logger->test == 1) {sprintf(logger->tmptxt, "Returning from backpro_d 3\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }

      clean(backproj->skyim_r, skyim, logger);		/* remove large scale structures in skyimage */
         
      /* ******************************************************** */
      /*                                                          */
      /*  Repeat source finding and elimination from shadowgram   */
      /*                                                          */
      /* ******************************************************** */
      
      goto igen;

nextee: 

   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return(nsrc);
}
