#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include <math.h>
#include <string.h>

#include "midi_b9.h"

double longspx[6][2], longspy[6][2], longspw, longspw2, longsph, offs_lsp; 
double shortspx[6][2], shortspy[6][2], shortspw, shortspw2, shortsph, offs_ssp; 
double ringri[3], ringro[3], ringh[3], rx[4]; 
double ringri2[3], ringro2[3]; 
double bolt1x[6], bolt1y[6], bolt1r, bolt1s, offs_b1, bolth; 
double bolt2x[6], bolt2y[6], bolt2r, bolt2s, offs_b2; 
double bolt3x[6], bolt3y[6], bolt3r, bolt3s, offs_b3; 
double joint1x[6], joint1y[6], joint1r, joint1h, joint1r2, offs_j1; 
double joint2x[6], joint2y[6], joint2r, joint2h, joint2r2, offs_j2;
double twist, r_twist, pi6;
unsigned char struk[1100][1100];

void strukdef(struct euclid *g_const, struct log_con *logger);
void s_turn(double xin, double yin, double angle, double *xout, double *yout, int prin, 
   struct euclid *g_const, struct log_con *logger);
void struktu(struct euclid *g_const, struct log_con *logger);
void bar(double endx[2], double endy[2], double w, int mark, 
   struct euclid *g_const, struct log_con *logger);
void ring(double ri, double ro, int mark, struct euclid *g_const, struct log_con *logger);
void bolt(double xc, double yc, double r, int mark, int prin, 
   struct euclid *g_const, struct log_con *logger);
int circle(double xc, double yc, double r, double x, double y, double *rv, struct log_con *logger);
void strucprint(double offx, double offy, struct log_con *logger);
void strucpr2(int step, int offx, struct log_con *logger);
int strukupdate(double xv, double yv, double rv, struct euclid *g_const, struct log_con *logger); 
   
/* *************************************************************** */

void struktu(struct euclid *g_const, struct log_con *logger)
{
   int i, j, k, mark;
   double endx[12][2], endy[12][2], w[12]; 
   double rir[3], ror[3], onax; 
   double xc[30], yc[30], r[30]; 
   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "struktu");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
   
   mark = 1;				/* mark for wide shadow border */
   ring(ringri2[0], ringro2[0], mark, g_const, logger);	/* ring 1 shadow region (check at 21 mm) */
   mark = 0;				/* mark for narrow shadow border */
   ring(ringri2[1], ringro2[1], mark, g_const, logger);	/* ring 2 shadow region (check at 10 mm) */
   ring(ringri2[2], ringro2[2], mark, g_const, logger);	/* ring 3 shadow region (check at 10 mm) */

   for (i=0; i<6; i++) {
      mark = 1;		/* mark for wide shadow border */
      bar(longspx[i], longspy[i], longspw2, mark, g_const, logger);
      bolt(bolt1x[i], bolt1y[i], bolt1s, mark, 1, g_const, logger);
      bolt(bolt2x[i], bolt2y[i], bolt2s, mark, 1, g_const, logger);
      mark = 0;		/* mark for narrow shadow border */
      bolt(bolt3x[i], bolt3y[i], bolt3s, mark, 1, g_const, logger);
      bar(shortspx[i], shortspy[i], shortspw2, mark, g_const, logger);
      bolt(joint1x[i], joint1y[i], joint1r2, mark, 1, g_const, logger);
      bolt(joint2x[i], joint2y[i], joint2r2, mark, 1, g_const, logger);
   }

   mark = 64;		/* mark for exclusion */
   ring(ringri[0], ringro[0], mark, g_const, logger);	/* mark ring 1 in struk array (excluded) */
   mark = 32;		/* mark for exclusion at 10 mm */
   ring(ringri[1], ringro[1], mark, g_const, logger);	/* mark ring 2 (exclude at 10 mm) */
   ring(ringri[2], ringro[2], mark, g_const, logger);	/* mark ring 3 (exclude at 10 mm) */
   
   for (i=0; i<6; i++) {
      mark = 64;		/* mark for exclusion */
      bar(longspx[i], longspy[i], longspw, mark, g_const, logger);
      bolt(bolt1x[i], bolt1y[i], bolt1r, mark, 1, g_const, logger);
      bolt(bolt2x[i], bolt2y[i], bolt2r, mark, 1, g_const, logger);

      mark = 32;		/* mark solid regions for exclusion at 10 mm) */
      bar(shortspx[i], shortspy[i], shortspw, mark, g_const, logger);
      bolt(joint1x[i], joint1y[i], joint1r, mark, 1, g_const, logger);
      bolt(joint2x[i], joint2y[i], joint2r, mark, 1, g_const, logger);
      bolt(bolt3x[i], bolt3y[i], bolt3r, mark, 1, g_const, logger);
   }

   k = 0;
   for (i=-250; i<250; i++) {   
      for (j=-250; j<250; j++) {   
         if ((i*i+j*j) > 62500) continue;
         if (struk[550+i][550+j] >= 32) k++;
      }
   }
   
   onax = (double)(k) / (3.14159 * 250.0 * 250.0);
   
/*   sprintf(logger->tmptxt, "\nOn-axis blocking by support structure: %6.3lf\n", onax);
   logger->logstat = logprint(logger->tmptxt, logger->logger->lognum);

      strucpr2(1, 0);
      strucpr2(1, 250);
      strucpr2(2, 0);
      strucpr2(2, 150);
      strucpr2(2, 300);
      strucpr2(2, 400);
      strucpr2(4, 0);
      strucpr2(8, 0);
*/	   																																		
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return;
}
	
/* *************************************************************** */

void bar(double endx[2], double endy[2], double w, int mark, 
   struct euclid *g_const, struct log_con *logger)
{
   int i, j;
   double length, xv, yv, dx, dy, r, rin, rout;
   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "bar");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
   
   length = sqrt((endx[0]-endx[1])*(endx[0]-endx[1]) +
      		 (endy[0]-endy[1])*(endy[0]-endy[1]));
   rin = sqrt(endx[0]*endx[0] + endy[0]*endy[0]);
   rout = sqrt(endx[1]*endx[1] + endy[1]*endy[1]);
/*   sprintf(logger->tmptxt, "Bar length: %7.2lf, rin: %7.2lf, rout: %7.2lf, endxy: %7.2lf/%7.2lf\n",
      		length, rin, rout, endx[0], endy[0]);
   logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
   r = w / 2.0;
   
   j = length / 0.5 + 1;
   dx = (endx[0]-endx[1]) / j;
   dy = (endy[0]-endy[1]) / j;
   for (i=0; i<=j; i++) {
      xv = endx[1] + i * dx;
      yv = endy[1] + i * dy;
      bolt(xv, yv, r, mark, 0, g_const, logger);
   }
   
/*   strucprint((endx[0]+endx[1])/2.0, (endy[0]+endy[1])/2.0);
*/
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return;
}
	
/* *************************************************************** */

void ring(double ri, double ro, int mark, struct euclid *g_const, struct log_con *logger)
{
   int ix, iy, xlow, xhigh, ylow, yhigh, n;
   double xv, yv, dn = 0.0, rc, rv;
   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "ring");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
   
   xlow = 550 - ro * 2.0;
   xhigh = 550 + ro * 2.0;
   ylow = 550 - ro * 2.0;
   yhigh = 550 + ro * 2.0;
   rc = (ri + ro) / 2.0;
   
   if (xlow < 0) xlow = 0;
   if (xhigh > 1099) xhigh = 1099;
   if (ylow < 0) ylow = 0;
   if (yhigh > 1099) yhigh = 1099;
   n = 0;
   
   for (ix=xlow; ix<=xhigh; ix++) {
      for (iy=ylow; iy<=yhigh; iy++) {
         xv = (double)(ix - 550) / 2.0;
         yv = (double)(iy - 550) / 2.0;
         if (circle(dn, dn, ro, xv, yv, &rv, logger) == 1) {
   	    if (rv > ri) {
               if (mark <= 1) struk[ix][iy] = strukupdate(xv, yv, rv, g_const, logger) + mark; 
               if (mark > 1) struk[ix][iy] = mark;
               n++;
            }
         }
      }
   }
/*   strucprint(rc, 0.0);
   strucprint(0.0, rc);
   strucprint(-rc, 0.0);
   strucprint(0.0, -rc);
*/
/*   sprintf(logger->tmptxt, "Ring: ri/ro: %6.2lf/%6.2lf, number of affected pixels: %6d,  mark: %1d\n", ri, ro, n, mark);
   logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return;
}
	
/* *************************************************************** */

void bolt(double xc, double yc, double r, int mark, int prin, 
   	struct euclid *g_const, struct log_con *logger)
{
   int ix, iy, ixlow, ixhigh, iylow, iyhigh, mrk;
   double xv, yv, c_angle, angle, rv, rb;
   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "bolt");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
   
   ixlow = (xc - r) * 2.0 + 550.0;
   ixhigh = (xc + r) * 2.0 + 550.0;
   iylow = (yc - r) * 2.0 + 550.0;
   iyhigh = (yc + r) * 2.0 + 550.0;

   if (ixlow < 0) ixlow = 0;
   if (ixhigh > 1099) ixhigh = 1099;
   if (iylow < 0) iylow = 0;
   if (iyhigh > 1099) iyhigh = 1099;
   for (ix=ixlow; ix<=ixhigh; ix++) {
      for (iy=iylow; iy<=iyhigh; iy++) {
         xv = (double)(ix - 550) / 2.0;
         yv = (double)(iy - 550) / 2.0;
         if (circle(xc, yc, r, xv, yv, &rb, logger) == 1) {
   	    if (mark > 1) struk[ix][iy] = mark;
            if (mark <= 1) {
               rv = sqrt(xv*xv + yv*yv);
               struk[ix][iy] = strukupdate(xv, yv, rv, g_const, logger) + mark; 
            }
         }
      }
   }
/*   if (prin == 1) strucprint(xc, yc);
*/
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return;
}

/* *************************************************************** */

int circle(double xc, double yc, double r, double x, double y, double *rv, struct log_con *logger)
{
   int inside;
   static int m=0;
   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "circle");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
   
   inside = 0;
   
   if ((*rv = sqrt((x-xc)*(x-xc) + (y-yc)*(y-yc))) <= r) inside = 1;
   m++;
   
/*   if (m < 1000) {
      sprintf(logger->tmptxt, 
         "Circle: m: %3d, xc/yc: %5.1lf/%5.1lf, r: %5.1lf, x/y: %6.1lf/%6.1lf, inside: %1d\n",
         m, xc, yc, r, x, y, inside);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
   }
*/   
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return(inside);
}

/* *************************************************************** */

void strukdef(struct euclid *g_const, struct log_con *logger)
{
   int i, j, s_offi, s_offj, prin;
   double ta, len;
   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "strukdef");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
   
/*   sprintf(logger->tmptxt, "Entering strukdef\n");
   logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
   twist = -4.286;		/* twist angle for entire support structure */
   r_twist = twist * g_const->gtorad;
   pi6 = g_const->pi / 6.0;
   for (i=0; i<1100; i++) for (j=0; j<1100; j++) struk[i][j] = 0;

   ringri[0] = 45.0;		/* inner radius of ring 1 */
   ringro[0] = 48.0;		/* outer radius of ring 1 */
   ringri2[0] = 45.0 - 3.5;	/* inner radius of shadow region */
   ringro2[0] = 48.0 + 3.5;	/* outer radius of shadow region */
   ringri[1] = 114.9;		/* inner radius of ring 2 */
   ringro[1] = 117.1;		/* outer radius of ring 2 */
   ringri2[1] = 114.9 - 1.7;	/* inner radius of shadow region */
   ringro2[1] = 117.1 + 1.7;	/* outer radius of shadow region */
   ringri[2] = 210.0;		/* inner radius of ring 3 */
   ringro[2] = 212.2;		/* outer radius of ring 3 */
   ringri2[2] = 210.0 - 1.7;	/* inner radius of shadow region */
   ringro2[2] = 212.2 + 1.7;	/* outer radius of shadow region */
   rx[0] = (ringri[0] + ringro[0]) / 2.0;
   rx[1] = (ringri[1] + ringro[1]) / 2.0;
   rx[2] = (ringri[2] + ringro[2]) / 2.0;
   rx[3] = 267.5;
   
   /* NOTE: the initial coordinates are not yet twisted by -4.286 degrees ! */
   longspx[0][0] = 46.5;	/* inner end of long spoke */
   longspx[0][1] = 268.0;	/* outer end of long spoke */
   longspy[0][0] = 0.0;		/* inner end of long spoke */
   longspy[0][1] = 0.0;		/* outer end of long spoke */
   longspw = 2.2;		/* width of long spoke */
   longspw2 = 2.2 + 7.0;	/* width of long spoke shadow */
   longsph = 21.25;		/* height of long spoke */
   offs_lsp = 30.0 + twist;	/* angular offset of first long spoke */

   shortspx[0][0] = 116.0;	/* inner end of short spoke */
   shortspx[0][1] = 211.0;	/* outer end of short spoke */
   shortspy[0][0] = 0.0;	/* inner end of short spoke */
   shortspy[0][1] = 0.0;	/* outer end of short spoke */
   shortspw = 2.2;		/* width of short spoke */
   shortspw2 = 2.2 + 3.4;	/* width of shadow region */
   shortsph = 10.25;		/* height of short spoke */
   offs_ssp = 0.0 + twist;	/* angular offset of first short spoke */

   bolt1x[0] = 49.5;		/* x coordinate of bolt1 */
   bolt1y[0] = 0.0;		/* y coordinate of bolt1 */
   bolt1r = 4.5;		/* radius of bolt1 */
   bolt1s = 4.5 + 3.5;		/* radius of bolt1 shadow */
   offs_b1 = 30.0 + twist;	/* angular offset of first bolt in 1. bolt set */

   bolt2x[0] = 116.0;	/* x coordinate of bolt2 */
   bolt2y[0] = 0.0;		/* y coordinate of bolt2 */
   bolt2r = 4.5;		/* radius of bolt */
   bolt2s = 4.5 + 3.5;		/* radius of bolt2 shadow */
   offs_b2 = 30.0 + twist;	/* angular offset of first bolt in 2. bolt set */
   
   bolt3x[0] = 211.4;	/* x coordinate of bolt3 */
   bolt3y[0] = 0.0;		/* y coordinate of bolt3 */
   bolt3r = 3.5;		/* radius of bolt */
   bolt3s = 3.5 + 3.5;		/* radius of bolt3 shadow */
   offs_b3 = 0.0 + twist;	/* angular offset of first bolt in 3. bolt set */
   
   joint1x[0] = 211.4;	/* x coordinate of joint1 */
   joint1y[0] = 0.0;		/* y coordinate of joint1 */
   joint1r = 3.34;		/* radius of joint1 */
   joint1r2 = 3.34 + 1.7;	/* outer radius of shadow region */
   offs_j1 = 30.0 + twist;	/* angular offset of first joint in 1. joint set */
   
   joint2x[0] = 117.1;	/* x coordinate of joint2 */
   joint2y[0] = 0.0;		/* y coordinate of joint2 */
   joint2r = 2.5;		/* radius of joint2 */
   joint2r2 = 2.5 + 1.7;	/* radius of shadow region */
   offs_j2 = 0.0 + twist;	/* angular offset of first joint in 2. joint set */
   
   for (j=1; j<7; j++) {  
      i = j % 6;
      ta = 60.0 * i;
/*      if ((i%3) == 0) prin = 1; else prin = 0;
*/
      s_turn(longspx[0][0], longspy[0][0], ta+offs_lsp, &longspx[i][0], &longspy[i][0], prin, g_const, logger);  
      s_turn(longspx[0][1], longspy[0][1], ta+offs_lsp, &longspx[i][1], &longspy[i][1], prin, g_const, logger);  
/*      len = sqrt((longspx[i][0]-longspx[i][1]) * (longspx[i][0]-longspx[i][1]) +
         	 (longspy[i][0]-longspy[i][1]) * (longspy[i][0]-longspy[i][1]));
      sprintf(logger->tmptxt, 
         "long %1d: x1/y1: %7.2lf/%7.2lf,  x2/y2: %7.2lf/%7.2lf,  length: %7.2lf\n", 
         i, longspx[i][0], longspy[i][0], longspx[i][1], longspy[i][1], len);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/
      s_turn(shortspx[0][0], shortspy[0][0], ta+offs_ssp, &shortspx[i][0], &shortspy[i][0], prin, g_const, logger);  
      s_turn(shortspx[0][1], shortspy[0][1], ta+offs_ssp, &shortspx[i][1], &shortspy[i][1], prin, g_const, logger);  
/*      len = sqrt((shortspx[i][0]-shortspx[i][1]) * (shortspx[i][0]-shortspx[i][1]) +
         	 (shortspy[i][0]-shortspy[i][1]) * (shortspy[i][0]-shortspy[i][1]));
      sprintf(logger->tmptxt, 
         "short %1d: x1/y1: %7.2lf/%7.2lf,  x2/y2: %7.2lf/%7.2lf,  length: %7.2lf\n", 
         i, shortspx[i][0], shortspy[i][0], shortspx[i][1], shortspy[i][1], len);
      
      logger->logstat = logprint(logger->tmptxt, logger->lognum);      
*/
      prin = 0;
      s_turn(bolt1x[0], bolt1y[0], ta+offs_b1, &bolt1x[i], &bolt1y[i], prin, g_const, logger);  
      s_turn(bolt2x[0], bolt2y[0], ta+offs_b2, &bolt2x[i], &bolt2y[i], prin, g_const, logger);  
      s_turn(bolt3x[0], bolt3y[0], ta+offs_b3, &bolt3x[i], &bolt3y[i], prin, g_const, logger);  
      s_turn(joint1x[0], joint1y[0], ta+offs_j1, &joint1x[i], &joint1y[i], prin, g_const, logger);  
      s_turn(joint2x[0], joint2y[0], ta+offs_j2, &joint2x[i], &joint2y[i], prin, g_const, logger);  
   }
   
   struktu(g_const, logger);	/* build structure image array */
   
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   

   return;
}

/* *************************************************************** */

void s_turn(double xin, double yin, double angle, double *xout, double *yout, int prin, 
   	struct euclid *g_const, struct log_con *logger)
{
   double sin_a, cos_a; 
   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "s_turn");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   

   sin_a = sin(angle*g_const->gtorad);
   cos_a = cos(angle*g_const->gtorad);
   
   *xout = cos_a * xin - sin_a * yin; 
   *yout = cos_a * yin + sin_a * xin; 
   
   if (prin == 1) {
      sprintf(logger->tmptxt, 
         "angle: %6.2lf, sin %6.4lf, cos %6.4lf, x/y in: %6.2lf/%6.2lf, x/y out: %6.2lf/%6.2lf\n", 
         	angle, sin_a, cos_a, xin, yin, *xout, *yout); 
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
   }
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return;
}

/* *************************************************************** */

void strucprint(double offx, double offy, struct log_con *logger)
{
   int i, j, s_offi, s_offj;
   char princhar;
   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "strukprint");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   

   s_offi = 550 + (int)(offy * 2.0 + 0.5);
   s_offj = 550 + (int)(offx * 2.0 + 0.5);
   if (s_offi < 21) s_offi = 21;
   if (s_offi > 1078) s_offi = 1078;
   if (s_offj < 64) s_offj = 64;
   if (s_offj > 1035) s_offj = 1035;

   sprintf(logger->tmptxt, 
      "offx: %7.1lf, offy: %7.1lf,  off_ix: %5d, off_iy: %5d\n\n", offx, offy, s_offi, s_offj);
   logger->logstat = logprint(logger->tmptxt, logger->lognum);

   for (i=21; i>=-21; i--) {
      logger->tmptxt[0] = 0;
      for (j=-64; j<=64; j++) {
         if (struk[s_offj+j][s_offi+i] < 10) princhar = '0' + struk[s_offj+j][s_offi+i];
         else princhar = 'A' + struk[s_offj+j][s_offi+i] - 10;
         sprintf(logger->tmptxt2, "%1c", princhar);
         strcat(logger->tmptxt, logger->tmptxt2);
/*         if (struk[s_offj+j][s_offi+i] > 0) 
   		sprintf(logger->tmptxt2, "%1x", struk[s_offj+j][s_offi+i]);
         else sprintf(logger->tmptxt2, ".");
*/
      }
      sprintf(logger->tmptxt2, "\n");
      strcat(logger->tmptxt, logger->tmptxt2);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
   }
   sprintf(logger->tmptxt, "\n");
   logger->logstat = logprint(logger->tmptxt, logger->lognum);
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
}   

/* *************************************************************** */

void strucpr2(int step, int off, struct log_con *logger)
{
   int i, j, s_offi, s_offj;
   char princhar;
   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "strukpr2");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   

   s_offi = 550 + off;
   s_offj = 550;
   if (s_offi < 64*step) s_offi = 64 * step;
   if (s_offi > (1099 - 64*step)) s_offi = 1099 - 64*step;
   if (s_offj < 64*step) s_offj = 64 * step;
   if (s_offj > (1099 - 64*step)) s_offj = 1099 - 64*step;

   sprintf(logger->tmptxt, 
      "offx: 0.0, offy: 0.0,  off_ix: 550, off_iy: 550,  step: %2d \n\n", step);
   logger->logstat = logprint(logger->tmptxt, logger->lognum);
   for (i=64*step; i>=-64*step; i-=step) {
      logger->tmptxt[0] = 0;
      for (j=-64*step; j<=64*step; j+=step) {
         if (struk[s_offj+j][s_offi+i] < 10) princhar = '0' + struk[s_offj+j][s_offi+i];
         else if (struk[s_offj+j][s_offi+i] < 64) princhar = 'A' + (struk[s_offj+j][s_offi+i] & 0x1F) - 10;
              else princhar = '';
         if (struk[s_offj+j][s_offi+i] == 0) {            
            princhar = ' ';
            if (((i/step) % 16) == 0) princhar = '-';
            if (((j/step) % 16) == 0) princhar = '|';
         }
         sprintf(logger->tmptxt2, "%1c", princhar);
         strcat(logger->tmptxt, logger->tmptxt2);
/*         if (struk[s_offj+j][s_offi+i] > 0) 
   		sprintf(logger->tmptxt, "%1x", struk[s_offj+j][s_offi+i]);
         else sprintf(logger->tmptxt, ".");
*/
      }
      sprintf(logger->tmptxt2, "\n");
      strcat(logger->tmptxt, logger->tmptxt2);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
   }
   sprintf(logger->tmptxt, "\n\n");
   logger->logstat = logprint(logger->tmptxt, logger->lognum);
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
}   

/* *************************************************************** */

int strukupdate(double xv, double yv, double rv, struct euclid *g_const, struct log_con *logger)

{
   int m6, m12;
    static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "strukupdate");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
  
   if (rv < rx[0]) {
      /* function exit timing start */
      TT_now = clock();
      logger->func_times[func_local] += difftime(TT_now, logger->TT);
      logger->TT = TT_now;
      logger->func_calling_num = local_calling_num;
      /* function exit timing end */
      return(0);
   }
   
   m12 = (atan2(yv, xv) + g_const->twopi - r_twist) / pi6;
   m12 %= 2;
   m6 = (atan2(yv, xv) + g_const->twopi - r_twist - pi6) / (pi6 * 2.0);
   m6 %= 2;

   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
/*   if (rv < rx[1]) return(8 + m6);
   if (rv < rx[2]) return(16 + m12);
   if (rv < rx[3]) return(32 + m6);
*/
   if (rv < rx[1]) return(2+2*m6);
   if (rv < rx[2]) return(8+8*m12);
   if (rv < rx[3]) return(2+2*m6);

   return(0);    
}

/* *************************************************************** */

int struchk(double xm, double ym, double x_det, double y_det, int last, struct euclid *g_const, 
   struct instr_data *jmx_id, struct log_con *logger)
/*	The "struk" array contains information about the mask support structure and its
        shadowing of the mask. The array covers a 550x550 mm2 area with pixels of 0.5x0.5 mm2.
        The access tests should be done in the following order:
        1) calculate intersection of ray with the mask plane. 
           Find corresponding pixel in the "struk" array.

        2) If the pixel value is 0 access is allowed along this path     (in open areas between structural members)
        3) If the pixel value is >= 32 access is forbidden along this path (hit solid structure or mask elim. hole)
        4) If the pixel value is >0 and <32:                (in potential shadow area close to a structural member)
           4a) If the pixel value is odd, 		             (close to a structural member of 21 mm height)
              Calculate intersection points of this photon with two leves at +/- 21.25 mm
           4b) If the pixel value is even, 		             (close to a structural member of 10 mm height)
              Calculate intersection points of this photon with two leves at +/- 10.25 mm
        5) If the pixel value in any of the two new points is 64 access is forbidden.         (hit solid structure)
        6) If the pixel value in any of the two new points is 32 
           			and the first pixel value was even, access is forbidden.      (hit solid structure)
        7) If the pixel value in both of the two new points is >= 0 
                                and the two values are different, access is forbidden.   (traverse solid structure)
        8) Otherwise access is allowed.

        The following symbols are used in the pixels of struk:
           0:	free access, no further tests are required.
           32:  access blocked by solid material in mask support structure of 10.25 mm height.
           64:  access blocked by solid material in mask support structure of 21.25 mm height.
           1, 3, 5, 7, 9: 17: Photon close to high structure. 
              		  Test required at plus/minus 21.25 mm from the mask plane.
           2, 4, 6, 8, 16: Photon close to low structure. 
              		  Test required at plus/minus 10.25 mm from the mask plane.
*/
{
   int i, j, ix, iy, ixb, iyb, ixu, iyu, sbval, suval;
   double xb, yb, xu, yu, dxl, dyl, dxh, dyh, xdif, ydif;
   static int p_count[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, first = 1;
   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "struchk");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
   
   if (first) {
         /* check that the mask_height has been defined (external variable) */
      if ((jmx_id->mask_height < 3390.0) || (jmx_id->mask_height > 3403.0)) {
         sprintf(logger->error_text, 
            "In m_strukdef: mask_height value outside acceptable range: %6.2lf\n", 
            jmx_id->mask_height);
         return(-1);
      }
      
      strukdef(g_const, logger);	/* initialize structure definitions */
      first = 0;
   }
   
   logger->lognum = 9;
   xb = xu = xm * 2.0 + 550.5; 
   yb = yu = ym * 2.0 + 550.5;
   
   xdif = xm - x_det;
   ydif = ym - y_det;
   
   dxl = xdif * 2.0 * shortsph / jmx_id->mask_height; 
   dyl = ydif * 2.0 * shortsph / jmx_id->mask_height; 
   dxh = xdif * 2.0 * longsph / jmx_id->mask_height; 
   dyh = ydif * 2.0 * longsph / jmx_id->mask_height; 
   
   ix = xb;
   iy = yb;
   
   if ((ix < 0) || (ix > 1099) || (iy < 0) || (iy > 1099)) {
      sprintf(logger->tmptxt, 
         "ix/iy outside limits: %4d/%4d, x/y_mask: %6.2f/%6.2f, x/y_det: %6.2f/%6.2f\n",
               ix, iy, xm, ym, x_det, y_det);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
      ix = iy = 0;   
   }
   i = 0;
   if (struk[ix][iy] == 0) {
      if (last) p_count[7]++;
      goto retu;	/* in open areas between structural members */
   }
   i = 1;
   if (struk[ix][iy] >= 32) {
      if (last) p_count[0]++;
      goto retu;	/* hit solid structure */
   }
   if ((struk[ix][iy]%2) == 0) {	/* struk value is even - close to a structural member of 10 mm height */
      xb -= dxl;
      yb -= dyl;
      ixb = xb;
      iyb = yb;

      if ((ixb < 0) || (ixb > 1099) || (iyb < 0) || (iyb > 1099)) {
         sprintf(logger->tmptxt, 
            "ixb/iyb outside limits: %4d/%4d, x/y_mask: %6.2f/%6.2f, x/y_det: %6.2f/%6.2f\n",
               ixb, iyb, xm, ym, x_det, y_det);
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
         ixb = iyb = 0;   
      }
      sbval = struk[ixb][iyb];
      if (sbval >= 32) {
         if (last) p_count[1]++;
         goto retu;	/* hit solid structure */
      }
   
      xu += dxl;
      yu += dyl;
      ixu = xu;
      iyu = yu;
      if ((ixu < 0) || (ixu > 1099) || (iyu < 0) || (iyu > 1099)) {
         sprintf(logger->tmptxt, 
            "ixu/iyu outside limits: %4d/%4d, x/y_mask: %6.2f/%6.2f, x/y_det: %6.2f/%6.2f\n",
               ixu, iyu, xm, ym, x_det, y_det);
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
         ixu = iyu = 0;   
      }
      suval = struk[ixu][iyu];

      if (suval >= 32) {
         if (last) p_count[2]++;
         goto retu;	/* hit solid structure */
      }
      if ((sbval > 0) && (suval > 0) && ((sbval&0xffe) != (suval&0xffe))) {
         if (last) p_count[3]++;
         goto retu;	/* traverse structure */
      }
   } else {				/* struk value is odd - close to a structural member of 21 mm height */

      xb = xu = xm * 2.0 + 550.5; 
      yb = yu = ym * 2.0 + 550.5;
   
      xb -= dxh;
      yb -= dyh;
      ixb = xb;
      iyb = yb;

      if ((ixb < 0) || (ixb > 1099) || (iyb < 0) || (iyb > 1099)) {
         sprintf(logger->tmptxt, 
            "ixb/iyb2 outside limits: %4d/%4d, x/y_mask: %6.2f/%6.2f, x/y_det: %6.2f/%6.2f\n",
               ixb, iyb, xm, ym, x_det, y_det);
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
         ixb = iyb = 0;   
      }
      sbval = struk[ixb][iyb];
      if (sbval >= 64) {
         if (last) p_count[4]++;
         goto retu;	/* hit solid structure */
      }
   
      xu += dxh;
      yu += dyh;
      ixu = xu;
      iyu = yu;
      if ((ixu < 0) || (ixu > 1099) || (iyu < 0) || (iyu > 1099)) {
         sprintf(logger->tmptxt, 
            "ixu/iyu2 outside limits: %4d/%4d, x/y_mask: %6.2f/%6.2f, x/y_det: %6.2f/%6.2f\n",
               ixu, iyu, xm, ym, x_det, y_det);
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
         ixu = iyu = 0;   
      }
      suval = struk[ixu][iyu];
      if (suval >= 64) {
         if (last) p_count[5]++;
         goto retu;	/* hit solid structure */
      }
      if ((sbval > 0) && (suval > 0) && ((sbval&0xffe) != (suval&0xffe))) {
         if (last) p_count[6]++;
         goto retu;	/* traverse structure */
      }
   }   
   if (last) p_count[8]++;
   i = 0;
retu:

   if (last < 2) {
      /* function exit timing start */
      TT_now = clock();
      logger->func_times[func_local] += difftime(TT_now, logger->TT);
      logger->TT = TT_now;
      logger->func_calling_num = local_calling_num;
      /* function exit timing end */
   
      if (i > 0) return(1); else return(0);
   }
   logger->lognum = 7;
   sprintf(logger->tmptxt, "Strukdef statistics:\n");
   logger->logstat = logprint(logger->tmptxt, logger->lognum);
   sprintf(logger->tmptxt, 
      " msk_level  low_under  low_over  low_trav  hgh_under  hgh_over  hgh_trav  init_OK  final_OK\n");
   logger->logstat = logprint(logger->tmptxt, logger->lognum);

   logger->tmptxt[0] = 0;
   for (j=0; j<9; j++) {
      sprintf(logger->tmptxt2, "%8d  ", p_count[j]);
      strcat(logger->tmptxt, logger->tmptxt2);
      p_count[j] = 0;
   }
   sprintf(logger->tmptxt2, "\n");
   strcat(logger->tmptxt, logger->tmptxt2);
   logger->logstat = logprint(logger->tmptxt, logger->lognum);

   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   if (i > 0) return(1); else return(0);
}

