/* Shadowgram backprojection. Niels Lund Dec. 2002.
Version 2.0 (minisky) Jan 2003 Using empirical empty field background maps and fixed sky-pixel source models.
Version 3.0 (midisky) Feb 2003 Using pixelfolding of shadowgrams and calculating source models according
                               to interpixel position determined by findpeak.
Uses pre-calculated shadowgrams "shadgram" in four energy bands. (prepared by "ipfread")  [0ymmdd_#shdJ.bin]
Uses a pre-calculated sky-backprojection file - with auxiliary data. (prepared by "skymapk")
Uses smoothed empty-field shadowgrams "soft_shdgr" in four energy bands. (prepared by "b_modelsmo")
Version 4.0 (midisky6-7) Uses pointing data to calculate sky cordinates.
                         (Programme "skyadjust.c" have been used to optimize conversion from 
                          instrument to sky-coordinates)
Version 5.0 (midisky8) Introduces lists of all source candidates seen any science window for a full file
   			and search for consistent localizations in this list (to catch low energy sources)
Version 6.0 (midiskyb, Aug. 2003) Take mask support structure and eliminated holes into account
midi_b3 Version 1.0 (Sep. 2003) Improve structure by splitting into separate files an eliminating dead code
midi_b4 Version 2.0 (Oct. 2003) Include generation of sky images in fits format
midi_b5 Version 1.0 (Nov. 2003) Include control data reading from files
midi_b5 Version 1.1 (March 2004) Export version with various minor updates & corrections
midi_b6 version 1.0 (June 2004) Include modelling of collimator deficiencies. Include common reading of alignment.dat
midi_b7 version 1.0 (June 2004) Include background models: 'empty field', 'particles', 'diffuse' (backmodels = 3)
                                Include rescaling of images to provide counts/cm2/s
midi_b7 version 1.1 (June 2004) !!!!!! 3 BCKGR MODELS NOT SUCCESSFULL. GO BACK TO 2 MODELS: 'empty field' + 'const'
midi_b7 version 1.2 (June 2004) Correct bug in 'clean' routine                                 
mb8     version 1.0 (August 2004) Unite midi_b7 (255x255 skymap) and maxi_b7 (511x511 skymap). 
mb8     version 1.0 (August 2004) Prepare for JEMIROS (ISDC interfaces and styles)
*/

#include "midi_b9.h"

struct instr_data JID;

struct euclid GC;

struct log_con LOG;

struct backpro BP;

/* !!! The following are backprojection related data */

/*
int      d_allow_p[detdim];
short    d_allow_l[detdim];
unsigned d_allow_st[detdim];
float    soft_gr[detdim];
int      pixel_st[detdim];
float    empty[4][detdim];
float    diffuse[detdim];

float    rsignif[skydim];
char     access[skydim];
float    fitsimage[4][5][skydim]
float    skyim_r[skydim];
int      illum[skydim];

int      o_ctrl;
 
double   empty_aux[30];
double   soft_tot[30];
double   parameters[20];

unsigned char pixel_list[500000];
*/

float soft_shdgr[4][detdim];
float soft_gr1[4][detdim];
double soft_tot1[30];
short shadgram[4][detdim];
double shdgr_aux[30];
double shdgr_aux1[30];


char ref_file_name[200]; 									

double parameters[20];

char notes[1000];	/* notes & comments for current skymap */

double x_rms[5] = {2.0, 1.5, 1.4, 1.3, 1.2}; 	/* position determination uncertainty at the boundaries*/
double y_rms[5] = {2.0, 1.5, 1.4, 1.3, 1.2}; 	/* the values given here are default values */
float rmsfact  = 1.0; 

double abs_len[5] = {0.1, 2.0, 4.3, 20.0, 80.0}; /* abs. length (mm) at the boundaries of the four energy bands */
                                                
float acc_w[4] = {0.2, 0.4, 0.6, 0.5};		/* rel. weight of low boundary contribution to events */

/* double abs_len[4] = {1.0, 4.0, 15.0, 55.0};	*/ /* absorbtion length (mm) in the four energy bands */
						/* new values for low energy version */
double accumT[2];

unsigned bb[32]; 

int aux_pidv, pha_sel[5];
int thisgroup = 0;
int yes2all = 0;

FILE *cat_fil, *srclistf, *srclist2;

/* **************************************************************************************** */

int backsky_init(int jemx, int dead_ano, char sky_version, int FOV_ff, char* full_ref_name, 
   	struct log_con *logger, struct backpro *backproj, struct instr_data *jmx_id, struct euclid *g_const);
   									/* initialize global parameters */
float peaktransfer(float x, float y, float imraw[skydim], float impeak[skydim], 
   	float *peakrate, struct backpro *backproj, struct log_con *logger);
void fi_headr(char fitsfil[], struct log_con *logger);
int alignment_init(int jemx, int FOV_ff, struct log_con *logger, struct instr_data *jmx_id,
   	struct backpro *backproj, struct euclid *g_const);

/* **************************************************************************************** */
/* **************************************************************************************** */


main(int argc, char* argv[])
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                                 */        
/*              mb6, Niels Lund, August 2004                       */        
/*                                                                 */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

{
   char iname[100], filnavn[200], dfilnavn[100], dummy[540], skymapf[100], fitsnavn[128];
   char dfilnavn1[100], resultxt[150], full_ref_name[200];
   unsigned char tmbyte[440];
   char sky_version = {'m'};
   
   float xx_g[2][5][sourcmax], yy_g[2][5][sourcmax], xx[sourcmax], yy[sourcmax];
   float goodbet_g[2][5][sourcmax], imgpeak[4][skydim], chkX, chkY;
   float norm_d[sourcmax], norm_d_g[2][5][sourcmax], backcnt[2][5][sourcmax];
   float final_s[2][sourcmax], norm_c[2][sourcmax], com_n[2][sourcmax];
   float com_avx[sourcmax], com_avy[sourcmax], com_sn[sourcmax], sumfun[sourcmax]; 
   float com_x[2][sourcmax], com_y[2][sourcmax], com_s[2][sourcmax];
   double p_RA[10000], p_dec[10000], p_posang[10000];   
   float shadow1[4][detdim], fk, sumsn, totsn, sn_best, sn_temp, temptot, tempgood;
   float tt, tt_old, tt_old2, sn_new, resid[detdim];
   float accum0x[accumax], accum1x[accumax], accum0y[accumax], accum1y[accumax], accums0[accumax];
   float accums1[accumax], accums[accumax];
   float accum2x[accumax], accum2y[accumax], accum3x[accumax], accum3y[accumax], acc2sn[accumax], acc3sn[accumax];
   float accu2RA[accumax], accu2dec[accumax], actot2[accumax], accums2[accumax];
   float actot0[accumax], actot1[accumax], accumRA[accumax], accumdec[accumax], accumx[accumax], accumy[accumax];
   float goodrms[2][5], valmin_rms, valmin_const, rms1, rms_old, cmx, cmy;
   float avr, cosdec, sumcnts, funval;
   float sumX, sumY, sumgood, src_chk_px, src_chk_py, x, y;
   float s_dec, s_RA, phaX, phaY, avX0, avY0, edge;
   float n_factor, nn_factor, peakval[8][sourcmax], peakrate, gnudat[200][4][3], sss, maxsss;
   float  shadow[detdim];

   int i, j, ii, n, status, apid, jemx, jemx_s, sky_index, value, length, nsrc, oline=0, chk_no;
   int selectnum, det_index, ee, lmax = 18, pointval, bitpix, dim1, dim2, f_ctrl, gnunum, maxjb;
   int iterations, nsrc_g[2][5];
   int k, m, accu=0, fillen, fillen_old = 0, filnum=0, j_1[sourcmax];
   int starTm, starTh, stopTh, stopTm, dead_ano, newpoint, n_common, n_common2, src_coord=0, iee;
   int goodnsrc[2][5], goodfits[5];
   int totcnt[2][5], e1, e2, eee[2][sourcmax], ksrc, jb, s_sum;
   int tours, q_plus, last_i, FOV_ff, jmxsign;
   double eval[5] = {2.4, 4.2, 8.4, 14.0, 35.0}; 
   int a_orbit[accumax], a_pid[accumax], a_pidv[accumax], accucode[accumax], jemxlocal;
   int mnum_g[2][5][sourcmax], accu2, accu3;
   int acc2pid[accumax], acc3pid[accumax], pidlast, orbit, pid, pidv, e_indic, acc3ee[accumax], ee0, ee1, ee2, ee3; 
   int acc3tot[accumax];

   short used[2][5][sourcmax], used1[20000], used2[20000], accumn0[accumax], accumn1[accumax], acc_i[accumax];
   short acc2ee[accumax], acc2jx[accumax];
   short shadow_r[detdim];
      
   double rms, rmsold, fitpwr, solut[sourcmax], limit_f, quality, qual_limit = 10.0;
   double solut_e[4][sourcmax], finalvar[4], sh_rms[5][4], sh_rms2;
   double startT, stopT, xrms, yrms, solutold; 
   double strength_g[2][sourcmax], sumtot, cdelt, rad_ang;

   double m_lst_ra[20000], m_lst_dec[20000], m_lst_x[20000], m_lst_y[20000], m_lst_s[20000], m_lst_T[20000];
   double m_lst_s0[20000], m_lst_s1[20000], m_lst_T0[20000], m_lst_T1[20000], m_lst_cnt[20000], m_lst_area[20000];
   double m_lst_q[20000];
   int m_lst_pid[20000], m_lst_e[20000], m_lst_jx[20000], m_lst_ee[20000], m_num=0;
   int s_reject[4] = {0, 0, 0, 0};
   int s_accept[4] = {0, 0, 0, 0};
   int p_count[10];

/*   float dd_dec, dd_RA, dd_dist, d_dec_GX340, d_RA_GX340;
   float dec_GX340 = -45.611, RA_GX340 = 251.449, dec_GX337 = -47.394, RA_GX337 = 248.507;
   float d_dec_GX337, d_RA_GX337, dist_GX340, dist_GX337;
   int arr_dec_GX340[20], arr_RA_GX340[20], arr_dist_GX340[30], arr_sn_GX340[30];
   int arr2_dec_GX340[20], arr2_RA_GX340[20], arr2_dist_GX340[30], arr2_sn_GX340[30];
   int arr_dec_GX337[20], arr_RA_GX337[20], arr_dist_GX337[30], arr_sn_GX337[30]; 
*/   
   FILE *in, *gnu_fil, *fillist, *chkfil;
   
   Fun fun_handle;
   
   struct instr_data *jmx_id   = {&JID};
   struct euclid     *g_const  = {&GC};
   struct log_con    *logger   = {&LOG};
   struct backpro    *backproj = {&BP};

   static int func_local = -1, local_calling_num=0;
   clock_t calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = 0;
      logger->func_used_num = 1;
      logger->func_calling_num = 0;
      sprintf(logger->func_names[func_local], "main");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   logger->TT = TT_now = clock();
   calling_TT = logger->TT;
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   /* function entry timing end */
   
   for (i=0; i<50; i++) logger->func_times[i] = 0.0;

   FOV_ff = FOV_f;
   backproj->FOV_radius = FOV_rad;
   backproj->backmodels = 2;
      
   logger->test = 0; /* NOTE: test is controlled from mb9_control.dat */

   if ( argc > 1) strcpy(iname, argv[1]) ;	/* list of shadowgram files, 4 shadowgrams for one JEM-X */
   else {
      sprintf(logger->tmptxt, 
         "Remember file_list! (Other control parameters must be in 'mb9_control.dat')\n");
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
      return;
   }
   
   cat_fil = fopen("accum.cat", "wt");

/*   sprintf (logger->sweethome, "/nfs/r6/jemx/nlwork/midi_distribution_2/work");
   sprintf (logger->sweethome2, "/nfs/r6/jemx/nlwork/midi_distribution_2/work/reference");
*/
   sprintf(logger->sweethome, "/home/nl/jemx/c_soft");
   sprintf(logger->sweethome2, "/home/nl/jemx/c_soft/reference");

   fprintf(cat_fil, "0  V10.4 20030603 corrected error in calculation of illuminated areas in 'imagefit'\n");
   fprintf(cat_fil, "0  V10.4 Modified calculation of SN (using only relevant fraction of the background)\n");
   fprintf(cat_fil, "0  V10.5 Include output of source model patterns  (midisky8) \n");
   fprintf(cat_fil, "0  V11 Include mask support structure and eliminated holes in modelling  (midiskyb) \n");
   fprintf(cat_fil, "0  V11 Uses sky version '%1c' with FOV_rad=%3d\n", sky_version, backproj->FOV_radius);
   fprintf(cat_fil, "0  midi_b V1. source code split into files, header file introduced. Uses sky version '%1c'\n", 
      sky_version);
   fprintf(cat_fil, "0  midi_b4 V2. uses 'funtools' to output image files in fits format\n");
   fprintf(cat_fil, "0  midi_b5 V1. uses instrument data input from '--/reference/alignment.dat'\n");
   fprintf(cat_fil, "0  midi_b5 V1. and control data input from 'midi_b5_control.dat'\n");
   fprintf(cat_fil, "0  midi_b6 V1. models non-ideal collimator and reads alignment data using 'alignment_init''\n");
   fprintf(cat_fil, "0  midi_b7 V1. new background modelling and control data input from 'midi_b7_control.dat'\n");
   fprintf(cat_fil, "0  midi_b7 V1. possibility to output images in counts/cm2/s\n");
   fprintf(cat_fil, "0  midi_b7 V1.2 correct bug in 'clean' routine." );
   fprintf(cat_fil, "0  midi_b7 V1.2 Get ref. background file from midi_b7_control.dat\n");
   fprintf(cat_fil, "0  mb8     V1.0 Combine midi_b7 & maxi_b7. Prepare for JEMIROS (ISDC interfaces)\n");
   fclose(cat_fil);
   
   cat_fil = fopen("accum.cat", "at");
   
   sprintf(resultxt, "mb9_control.dat");
   in = fopen(resultxt, "rt");
   if (in != NULL) goto ct_sta;
   
   sprintf(resultxt, "%s/reference/mb9_control.dat", logger->sweethome);
   in = fopen(resultxt, "rt");
   if (in == NULL) {
      sprintf(logger->tmptxt, "No control data file found! (%s)\n", resultxt);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
      return;
   } 
ct_sta:      
   sprintf(logger->tmptxt, "0 Control data file: %s \n", resultxt);
   logger->logstat = logprint(logger->tmptxt, logger->lognum);
   for (i=0; i<200; i++) {
      fgets(resultxt, 150, in);
      if (feof(in)) goto x_end;
      fprintf(cat_fil, "0 Control data file: %s", resultxt);
      sprintf(logger->tmptxt, "%s", resultxt);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
      sscanf(resultxt, "%d", &n);
      if (n == 1) sscanf(resultxt, "%d %d", &n, &dead_ano);
      if (n == 2) sscanf(resultxt, "%d %d", &n, &iterations);
      if (n == 3) sscanf(resultxt, "%d %f", &n, &edge);
      if (n == 4) sscanf(resultxt, "%d %lf", &n, &limit_f);
      if (n == 5) sscanf(resultxt, "%d %lf", &n, &qual_limit);
      

      if (n == 100) sscanf(resultxt, "%d %d", &n, &logger->o_ctrl);
      if (n == 101) sscanf(resultxt, "%d %d", &n, &f_ctrl);
      if (n == 102) sscanf(resultxt, "%d %d", &n, &logger->test);
      if (n == 103) sscanf(resultxt, "%d %s", &n, ref_file_name);
      if (n == 104) sscanf(resultxt, "%d %d", &n, &yes2all);
   }
x_end:
   fclose(in);
   
   sprintf(logger->tmptxt, 
      "dead_ano: %d, iterations: %d, edge: %f, limit_f: %lf, qual_limit: %lf,  o_ctrl: %d, f_ctrl: %d\n",
      dead_ano, iterations, edge, limit_f, qual_limit, logger->o_ctrl, f_ctrl);
   logger->logstat = logprint(logger->tmptxt, logger->lognum);
   
   g_const->pi = 3.14159;
   g_const->twopi = g_const->pi * 2.0;
   g_const->gtorad = g_const->pi / 180.0;
   
   tt = tt_old = 345.0;

   jemx = 0;		/* default: JEM-X1 (jemx = 0) */

   fillist = fopen(iname,"rt") ;	/* opening file with list of shadowgrams */
   if (fillist == NULL) {
      sprintf(logger->tmptxt, "no %s file, exiting!\n", iname);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
      exit(0);
   }
   
   filnum = 0;
   
/*   sprintf(logger->tmptxt, "Opened files\n"); 
   logger->logstat = logprint(logger->tmptxt, logger->lognum);
*/

   if (logger->o_ctrl > 0) logger->o_ctrl = 1;
 
next_file:
   fgets(dfilnavn, 100, fillist);
   if (feof(fillist)) goto final;
   fillen = strlen(dfilnavn);
   if (filnum > 0) if (fillen < fillen_old) goto final;
   filnum++;
   fillen_old = fillen;
   
   dfilnavn[fillen-1] = 0;
   

   sprintf(logger->tmptxt, "\n\n **********************\n\nReady to read shadowgram file from %s\n", dfilnavn);
   logger->logstat = logprint(logger->tmptxt, logger->lognum);

   in = fopen(dfilnavn,"rb") ;		/* opening shadowgram file, 4 shadowgrams for one JEM-X */
   if ( in == NULL) {
      sprintf(logger->tmptxt, "Error opening file: %s ..... exiting!  fillen %3d\n", dfilnavn, fillen);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
      return;
   }
   
   fread(shadgram, 2, 4*detdim, in);
   fread(shdgr_aux, 8, 30, in);
   fclose(in);
   
   pha_sel[0] = shdgr_aux[17];
   pha_sel[1] = shdgr_aux[18];
   pha_sel[2] = shdgr_aux[19];
   pha_sel[3] = shdgr_aux[20];
   pha_sel[4] = shdgr_aux[21];
   jemx = shdgr_aux[22]; 
   backproj->aux_orbit = shdgr_aux[4] / 65536.0 + 0.01;

   for (i=0; i<30; i+=3) {
      sprintf(logger->tmptxt, "shdg_aux[%2d/%2d/%2d]: %12.3lf,  %12.3lf,  %12.3lf\n", 
         i, i+1, i+2, shdgr_aux[i], shdgr_aux[i+1], shdgr_aux[i+2]);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
   }

   if (jemx == 1) {
      if ((backproj->aux_orbit >= 26) && (backproj->aux_orbit <= 44)) ref_file_name[8] = '6'; 
      if ((backproj->aux_orbit >= 45) && (backproj->aux_orbit <= 102)) ref_file_name[8] = '7'; 
      if ((backproj->aux_orbit >= 103) && (backproj->aux_orbit <= 170)) ref_file_name[8] = '8'; 
   }
   sprintf(full_ref_name, "%s/reference/%s_%1d.bin", logger->sweethome, ref_file_name, jemx+1);

   status = backsky_init(jemx, dead_ano, sky_version, FOV_ff, full_ref_name, logger, backproj, jmx_id, g_const);
   if (status < 0) {
      sprintf(logger->tmptxt, "Bad status on return from backsky_init\n"); 
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
      return;
   }

   if ((pha_sel[0] != (int)(backproj->empty_aux[17])) || 
       (pha_sel[1] != (int)(backproj->empty_aux[18])) || 
       (pha_sel[2] != (int)(backproj->empty_aux[19])) ||
       (pha_sel[3] != (int)(backproj->empty_aux[20])) || 
       (pha_sel[4] != (int)(backproj->empty_aux[21]))) {
      sprintf(logger->tmptxt, "WARNING: Event selection criteria do match in data and reference shadowgrams\n");
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
      sprintf(logger->tmptxt, "Shadowgram data from: %s,  pha_sel:  %3d  %3d  %3d  %3d  %3d\n", 
         dfilnavn, pha_sel[0], pha_sel[1], pha_sel[2], pha_sel[3], pha_sel[4]);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
      sprintf(logger->tmptxt, "Reference data from:  %s,  pha_sel:  %3.0lf  %3.0lf  %3.0lf  %3.0lf  %3.0lf\n", 
         full_ref_name, backproj->empty_aux[17], backproj->empty_aux[18], 
         backproj->empty_aux[19], backproj->empty_aux[20], backproj->empty_aux[21]);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
      if (yes2all == 1) goto go_on;

      sprintf(logger->tmptxt, "Do you want to continue? (y/n):\n");
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
      if (getchar() == (int)('g')) {
         yes2all = 1; 
         goto go_on;
      }
      if (getchar() != (int)('y')) return;
      sprintf(logger->tmptxt, "OK, continuing!\n");
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
   } 
go_on:   
   eval[0] = backproj->empty_aux[4];
   eval[1] = backproj->empty_aux[5];
   eval[2] = backproj->empty_aux[8];
   eval[3] = backproj->empty_aux[11];
   eval[4] = backproj->empty_aux[14];

   if (filnum == 1) {
      fprintf(cat_fil, "0  ph_boundaries (ch):        %6d   %6d   %6d   %6d   %6d\n", 
            pha_sel[0], pha_sel[1], pha_sel[2], pha_sel[3], pha_sel[4]);
      fprintf(cat_fil, "0  absorbtion length (mm):     %5.2lf  %5.2lf  %5.2lf  %5.2lf\n", 
            abs_len[0], abs_len[1], abs_len[2], abs_len[3], abs_len[4]);
      fprintf(cat_fil, "0  x_rms scatter (mm):         %5.2lf  %5.2lf  %5.2lf  %5.2lf %5.2lf\n", 
            x_rms[0], x_rms[1], x_rms[2], x_rms[3], x_rms[4]);
      fprintf(cat_fil, "0  x_rms scatter (mm):         %5.2lf  %5.2lf  %5.2lf  %5.2lf %5.2lf\n", 
            y_rms[0], y_rms[1], y_rms[2], y_rms[3], y_rms[4]);
      fprintf(cat_fil, "0  energy limits (keV):        %5.1f   %5.1f   %5.1f   %5.1f  %5.1f\n", 
            eval[0], eval[1], eval[2], eval[3], eval[4]);
      sprintf(logger->tmptxt, "ref_file: %s, energy limits: %5.1f %5.1f %5.1f %5.1f %5.1f\n", 
            ref_file_name, eval[0], eval[1], eval[2], eval[3], eval[4]);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
   }
   accumT[jemx] = shdgr_aux[27];
   
   fprintf(cat_fil, "0 Shadowgram data read from %s,  accum_T: %8.1lf s\n", dfilnavn, accumT[jemx]);

      /* shdgr_aux[4] = Orbit_number * 65536 + Pointing ID */
   backproj->aux_orbit = shdgr_aux[4] / 65536.0 + 0.01;
   backproj->aux_pid = shdgr_aux[4] - 65536.0 * backproj->aux_orbit + 0.01;
   aux_pidv = shdgr_aux[28];
   if (backproj->aux_pid < 10000) 
      pointval = getRAdec_init(jemx, p_RA, p_dec, p_posang, g_const, logger, jmx_id, backproj);
   else  {
      sprintf(logger->tmptxt, " Warning: pid value exceeds array dimension!!!: %4d\n", backproj->aux_pid);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
   }


      /* ******************************************************** */
      /*                                                          */
      /*      Here starts the loop over different energy bands    */
      /*                                                          */
      /* ******************************************************** */

   for (ee=0; ee<4; ee++) {	/* ee = 0, 1, 2, 3 indicates the energy band */

	/* transfer the raw shadowgram to be analyzed into "shadow_r[]" */
      s_sum = s_accept[ee] = s_reject[ee] = 0;
      for (i=0; i<detdim; i++) {
         backproj->soft_gr[i] = soft_shdgr[ee][i];
         if (backproj->soft_gr[i] > 0.0) {
            shadow_r[i] = shadgram[ee][i]; 
            s_accept[ee] += shadgram[ee][i];
            s_sum += shadow_r[i]; 
         } else {
            shadow_r[i] = 0;
            s_reject[ee] += shadgram[ee][i];
         }
      }
      fprintf(cat_fil, "0  Accepted and rejected shadowgram counts: E%1d: %8d  %8d\n", 
         ee, s_accept[ee], s_reject[ee]);

      sh_rms[ee][0] = rmscalc_i(shadow_r, backproj->soft_gr, logger);
      
	/* pixelfold generates a shadowgram "shadow" which is corrected for detector nonlinearities */   
      totcnt[jemx][ee] = pixelfold(shadow_r, ee, shadow, logger, backproj, jmx_id);

      if (totcnt[jemx][ee] == 0) {
         nsrc_g[jemx][ee] = backproj->backmodels;
         continue;
      }
      
      sh_rms[ee][1] = rmscalc(shadow, backproj->soft_gr, logger);
      
      
      fk = 0.0;
      j = 0;
      for (i=0; i<detdim; i++) {
         if (backproj->soft_gr[i] > 0.0) fk += shadow[i]; 
         if (shadow[i] > 0.0) j++;
         backproj->shadw[i] = shadow[i];
      }
      
      sprintf(logger->tmptxt, 
         "JEM-X%1d. Cnts/rms before pixelfold: %6d/%8.2lf, after: %6d/%8.2lf, check: %6.0f, E%1d, nonzero: %5d\n", 
         jemx+1, s_sum, sh_rms[ee][0], totcnt[jemx][ee], sh_rms[ee][1], fk, ee, j);
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
      sprintf(logger->tmptxt, "J");

      if (totcnt[jemx][ee] <= 0) goto next_file;

      /*jmx_id->*/
      jmx_id->absolen = abs_len[ee];

         /* calculate the position scatter around the X-ray absorbtion point at this energy */
      jmx_id->scatt_len = sh_xy(x_rms[ee], y_rms[ee], logger, jmx_id);

      /* *********************************************************************** */
      /*                                                                         */
      /*  The "irosloop" go through the full "Iterative Removal Of Sources"      */
      /*  sequence until the loop terminates due to limited significance or      */
      /*  too many iteration loops                                               */
      /*                                                                         */
      /* *********************************************************************** */

      if (logger->test == 1) { 
         sprintf(logger->tmptxt, "calling irosloop\n"); 
         logger->logstat = logprint(logger->tmptxt, 3);
      }   
      nsrc = irosloop(jemx, ee, xx, yy, norm_d, solut, sumfun, &goodrms[jemx][ee], &sh_rms2, 
         	&goodfits[ee], &goodnsrc[jemx][ee], resid, edge, goodbet_g[jemx][ee], iterations, 
                limit_f, g_const, jmx_id, logger, backproj);
      if (nsrc < 0) {
         logger->logstat = logprint(logger->error_text, logger->lognum);
         return;
      }   
                
      if (logger->test == 1) {
         sprintf(logger->tmptxt, "returning from irosloop\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum);
      }   
   

      if (nsrc < 0) {
         n_common = n_common2 = 0;
         goto next_file;
      }
      sh_rms[ee][2] = sh_rms2;
                 
nextee:	
      /* ******************************************************** */
      /*                                                          */
      /*  Analysis of this energy band completed. Store results   */
      /*  for later comparison with the other energy bands.       */
      /*                                                          */
      /* ******************************************************** */
      
      sprintf(logger->tmptxt, "\n");
      logger->logstat = logprint(logger->tmptxt, logger->lognum);
         
      dfilnavn[17] = 0;
      fprintf(srclistf, "%s, jemx: %1d,  E: %d, t: %6.0lf s, Const. fitpwr, %8.1lf, Background fitpwr, %8.1lf\n",
         dfilnavn, jemx+1, ee, accumT[jemx], solut[0], solut[1]);

      nsrc_g[jemx][ee] = nsrc;

      for (i=0; i<nsrc; i++) { 

         xx_g[jemx][ee][i] = xx[i];
         yy_g[jemx][ee][i] = yy[i];
         if (i >= backproj->backmodels) norm_d_g[jemx][ee][i] = 0.01 / (norm_d[i] * sumfun[i]); 

         if ((pointval > 0) && (i >= backproj->backmodels)) {
                        
            	/* get the background area corresponding to each source */
            status = fun(i, 0, ee, xx, yy, jemx, norm_d, 0, &funval, g_const, jmx_id, logger, backproj);
            if (status < 0) {
               logger->logstat = logprint(logger->error_text, logger->lognum); 
               return;
            }
            
            backcnt[jemx][ee][i] = 2.0 * funval;
            
/*            sprintf(logger->tmptxt, "Src: %2d, E: %1d, Background estimate: %8.1f, cm2: %6.2f, total: %6d\n",
                  i, ee, backcnt[jemx][ee][i], 0.01/norm_d[i], totcnt[jemx][ee]);
            logger->logstat = logprint(logger->tmptxt, logger->lognum); 
*/

   	/* determine RA and dec of the source candidates found at this energy */

            getRAdec(xx[i], yy[i], jemx, &s_RA, &s_dec, g_const, logger, jmx_id, backproj);
           
            fprintf(srclistf, "src: %1d, fitpwr: %8.1lf, XY: %+6.1f %+6.1f  E: %2d  RA/dec: %+7.3f %+7.3f \n", 
               i, solut[i], xx[i], yy[i], ee, s_RA, s_dec); 
            sprintf(logger->tmptxt, 
               "src: %1d, fitpwr: %8.1lf, XY: %+6.1f %+6.1f  E: %2d  RA/dec: %+7.3f %+7.3f\n", 
               i, solut[i], xx[i], yy[i], ee, s_RA, s_dec); 
            logger->logstat = logprint(logger->tmptxt, logger->lognum); 

               /* establish data list for all candidates found - ought to be coded as a C-structure */
               
            m_lst_ra[m_num] = s_RA;
            m_lst_dec[m_num] = s_dec;
            m_lst_pid[m_num] = backproj->aux_orbit*100000 + backproj->aux_pid * 100 + aux_pidv;
            m_lst_x[m_num] = xx[i];
            m_lst_y[m_num] = yy[i];
            m_lst_s[m_num] = solut[i] * solut[i];
            m_lst_e[m_num] = ee;
            m_lst_jx[m_num] = jemx+1;
            m_lst_ee[m_num] = ee;
            m_lst_T[m_num] = accumT[jemx];
            m_lst_s0[m_num] = solut[0] * solut[0];
            m_lst_s1[m_num] = solut[1] * solut[1];
            m_lst_T0[m_num] = shdgr_aux[23];
            m_lst_T1[m_num] = shdgr_aux[24];
            m_lst_cnt[m_num] = backcnt[jemx][ee][i]; 
            m_lst_q[m_num] = goodbet_g[jemx][ee][i] / sqrt((double)(totcnt[jemx][ee])); 

            if (i >= backproj->backmodels) m_lst_area[m_num] = norm_d_g[jemx][ee][i];
            else  m_lst_area[m_num] = 0.01 * backproj->soft_tot[26+ee];

            m_num++;

            if (m_num > 19999) m_num = 19999;
         } else {
            if (i >= backproj->backmodels) fprintf(srclistf, "src: %1d, fitpwr: %8.1lf, XY: %+6.1f %+6.1f  E: %2d\n", 
               i, solut[i], xx[i], yy[i], ee);
         }
      }

      for (i=0; i<detdim; i++) {
         shadow1[ee][i] = shadow[i];
         soft_gr1[ee][i] = backproj->soft_gr[i];
      }
   }		/* end of loop over 4 energy bands */

   for (iee=0; iee<4; iee++) {

      if (totcnt[jemx][iee] == 0.0) continue;

      sprintf(logger->tmptxt, 
         "\nngrp[%1d]: %d, Best srcnum: %2d, best/final fitcount: %2d, best rms: %6.3f\n",
         iee, nsrc_g[jemx][iee]-backproj->backmodels, goodnsrc[jemx][iee], goodfits[iee], goodrms[jemx][iee]);
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      sprintf(logger->tmptxt, "Best fit coefficients    : ");
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      for (j=0; j<nsrc_g[jemx][iee]-1; j++) {
         sprintf(logger->tmptxt, " %7.1f", goodbet_g[jemx][iee][j]);
         logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      }
      sprintf(logger->tmptxt, "\n");
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      sprintf(logger->tmptxt, "Corresp. eff. areas (cm2): ");
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      for (j=0; j<nsrc_g[jemx][iee]-1; j++) {
         sprintf(logger->tmptxt, " %7.1f", norm_d_g[jemx][iee][j]);
         logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      }
      sprintf(logger->tmptxt, "\n");
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      sprintf(logger->tmptxt, 
         "Integration time: %8.1lf (s),  Absorbtion length: %5.2lf (mm)\n", accumT[jemx], abs_len[iee]);
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
   }


      /* ******************************************************** */
      /*                                                          */
      /*    print of combined results from several energy bands   */
      /*                                                          */
      /* ******************************************************** */
      
   startT = (shdgr_aux[23] - (int)(shdgr_aux[23])) * 24.0;
   starTh = startT;
   starTm = (startT - starTh) * 60;
   stopT = (shdgr_aux[24] - (int)(shdgr_aux[24])) * 24.0;
   stopTh = stopT;
   stopTm = (stopT - stopTh) * 60;

   for (i=0; i<sourcmax; i++) {
         used[jemx][0][i] = used[jemx][1][i] = used[jemx][2][i] = used[jemx][3][i] = 0;
         used2[i] = 0;
   }

   sprintf(logger->tmptxt, "\nNumber of sources: %2d %2d %2d %2d:\n",      
         nsrc_g[jemx][0]-backproj->backmodels, nsrc_g[jemx][1]-backproj->backmodels, 
         nsrc_g[jemx][2]-backproj->backmodels, nsrc_g[jemx][3]-backproj->backmodels); 
   logger->logstat = logprint(logger->tmptxt, logger->lognum); 
         
   k = 0;
   for (e1=0; e1<3; e1++) {
         if (nsrc_g[jemx][e1] > backproj->backmodels) { 
            for (i=backproj->backmodels; i<nsrc_g[jemx][e1]; i++) {
            
               if (used[jemx][e1][i] > 0) continue;
               
               quality = (double)(goodbet_g[jemx][e1][i]) / 
                  	sqrt((double)(totcnt[jemx][e1] * norm_d_g[jemx][e1][i] * 100.0 / backproj->soft_tot[26+e1]));

               if (quality < 1.0) continue; 
   	       	
               sumX = xx_g[jemx][e1][i] * goodbet_g[jemx][e1][i];
               sumY = yy_g[jemx][e1][i] * goodbet_g[jemx][e1][i];
               sumgood = sumsn = goodbet_g[jemx][e1][i];
               sumtot = totsn = backcnt[jemx][e1][i];
               sn_best = sumgood / sqrt((double)(sumtot));
                  
               for (e2=e1+1; e2<4; e2++) {
                  
                  if ((nsrc_g[jemx][e2]-backproj->backmodels) <= 0) continue;
                  for (j=backproj->backmodels; j<nsrc_g[jemx][e2]; j++) {
                     if (used[jemx][e2][j] > 0) continue;
                     if (goodbet_g[jemx][e2][j] < 
                  	sqrt((double)(totcnt[jemx][ee] * norm_d_g[jemx][ee][j] * 100.0 / backproj->soft_tot[26+ee]))) 
                           continue; 

                     if (sqrt((double)((xx_g[jemx][e1][i]-xx_g[jemx][e2][j])*(xx_g[jemx][e1][i]-xx_g[jemx][e2][j]) + 
                           (yy_g[jemx][e1][i]-yy_g[jemx][e2][j])*(yy_g[jemx][e1][i]-yy_g[jemx][e2][j]))) < 1.5) { 

                        tempgood = sumsn + goodbet_g[jemx][e2][j];
                        temptot = totsn + backcnt[jemx][e2][j];
                        sn_temp = tempgood / sqrt((double)(temptot));
                        sn_new = goodbet_g[jemx][e2][j] / sqrt((double)(backcnt[jemx][e2][j]));

                        if ((sn_temp >= sn_best) && (sn_temp >= sn_new )) {
                           sn_best = sn_temp;
                           sumsn = tempgood;
                           totsn = temptot;
                        }
                        if ((sn_new  >= sn_best) && (sn_new  >= sn_temp)) {
                           sn_best = sn_new;
                           sumsn = goodbet_g[jemx][e2][j];
                           totsn = totcnt[jemx][e2];
                        }
                        
                        sumgood += goodbet_g[jemx][e2][j];
                        
                        sumX += xx_g[jemx][e2][j] * goodbet_g[jemx][e2][j];
                        sumY += yy_g[jemx][e2][j] * goodbet_g[jemx][e2][j];
                        
                        avX0 = sumX / sumgood;
                        avY0 = sumY / sumgood;
                      
                        logger->tmptxt[0] = 0; 
                        if (pointval > 0) {
                        
                           getRAdec(avX0, avY0, jemx, &s_RA, &s_dec, g_const, logger, jmx_id, backproj);

                           sprintf(logger->tmptxt2, "%s RA/dec: %+7.3f %+7.3f  fit: %1d %7.1lf %1d %7.1lf ",
                              dfilnavn, s_RA, s_dec, e1, goodbet_g[jemx][e1][i], e2, goodbet_g[jemx][e2][j]);
                           strcat(logger->tmptxt, logger->tmptxt2);
                           
                           fprintf(srclistf, "%s  RA/dec: %+7.3f %+7.3f  fit: %1d %7.1lf %1d %7.1lf ",
                              dfilnavn,  s_RA, s_dec, e1, goodbet_g[jemx][e1][i], e2, goodbet_g[jemx][e2][j]);
                           
                        } else {
                           sprintf(logger->tmptxt2, "%s  XY: %+7.1f %+7.1f  strengths: %7.1lf %7.1lf ",
                              dfilnavn, avX0-(float)(backproj->FOV_radius), avY0-(float)(backproj->FOV_radius), 
                              goodbet_g[jemx][e1][i], goodbet_g[jemx][e2][j]);
                           strcat(logger->tmptxt, logger->tmptxt2);

                           fprintf(srclistf, "%s  XY: %+7.1f %+7.1f  strengths: %7.1lf %7.1lf ",
                              dfilnavn, avX0-(float)(backproj->FOV_radius), avY0-(float)(backproj->FOV_radius), 
                              goodbet_g[jemx][e1][i], goodbet_g[jemx][e2][j]);
                        }
                        
                        if ((starTh + starTm + stopTh + stopTm) == 0) {
                           sprintf(logger->tmptxt2, "\n");
                        } else {
                           sprintf(logger->tmptxt2, "time: %2d:%2d to %2d:%2d\n", starTh, starTm, stopTh, stopTm);
                        }
                        strcat(logger->tmptxt, logger->tmptxt2);
                        logger->logstat = logprint(logger->tmptxt, logger->lognum); 
                        
                        if ((starTh + starTm + stopTh + stopTm) == 0) fprintf(srclistf, "\n");
                        else fprintf(srclistf, "time: %2d:%2d to %2d:%2d\n", starTh, starTm, stopTh, stopTm);

                        used[jemx][e2][j] = 1;
                        used2[i] = 1;
                        accum0x[accu] = xx_g[jemx][e1][i]; 
                        accum0y[accu] = yy_g[jemx][e1][i]; 
                        accum1x[accu] = xx_g[jemx][e2][j]; 
                        accum1y[accu] = yy_g[jemx][e2][j];
                        a_orbit[accu] = backproj->aux_orbit;
                        a_pid[accu] = backproj->aux_pid;
                        a_pidv[accu] = aux_pidv;
                        accums0[accu] = goodbet_g[jemx][e1][i];
                        accums1[accu] = goodbet_g[jemx][e2][j];
                        actot0[accu] = backcnt[jemx][e1][i];
                        actot1[accu] = backcnt[jemx][e2][j];
                        accucode[accu] = jemx * 10000 + e2 * 100 + e1;
                        accumRA[accu] = s_RA;
                        accumdec[accu] = s_dec;
                        accumx[accu] = avX0;
                        accumy[accu] = avY0;
                        accums[accu] = sn_best;
                        accumn0[accu] = mnum_g[jemx][e1][i]; 
                        accumn1[accu] = mnum_g[jemx][e2][j]; 

                        com_x[0][k] = xx_g[jemx][e1][i]; 
                        com_y[0][k] = yy_g[jemx][e1][i];
                        norm_c[0][k] = norm_d_g[jemx][e1][i];
                        norm_c[1][k] = norm_d_g[jemx][e2][j];
                        com_x[1][k] = xx_g[jemx][e2][j]; 
                        com_y[1][k] = yy_g[jemx][e2][j];
                        com_s[0][k] = goodbet_g[jemx][e1][i];
                        com_s[1][k] = goodbet_g[jemx][e2][j];
                        com_avx[k] = avX0;
                        com_avy[k] = avY0;
                        com_sn[k] = sn_best;
                        backproj->com_RA[k] = s_RA;
                        backproj->com_dec[k] = s_dec;
                        com_n[0][k] = mnum_g[jemx][e1][i]; 
                        com_n[1][k] = mnum_g[jemx][e2][j]; 
                        eee[0][k] = e1;
                        eee[1][k] = e2;
                        sprintf(logger->tmptxt, "Energy scales: %d %d, n: %d, XY0: %6.1f %6.1f, XY1: %6.1f %6.1f \n", 
                           e1, e2, k, accum0x[accu], accum0y[accu], accum1x[accu], accum1y[accu]);
                        logger->logstat = logprint(logger->tmptxt, logger->lognum); 
                     }
                  }
               }
               
               if (used2[i] > 0) {
                  used[jemx][e1][i] = 1; 
                  accu++;
                  if (accu >= accumax) accu = accumax-1;
                  k++;
               }
               used2[i] = 0;
            }
         }
   }
   fprintf(srclistf, "\n");
   sprintf(logger->tmptxt, "\n");
   logger->logstat = logprint(logger->tmptxt, logger->lognum); 

   n_common = k;

   for (e1=0; e1<4; e1++) {
         if (nsrc_g[jemx][e1] > backproj->backmodels) { 
            for (i=backproj->backmodels; i<nsrc_g[jemx][e1]; i++) {
            
               if (used[jemx][e1][i] > 0) continue;
               
               quality = (double)(goodbet_g[jemx][e1][i]) / sqrt((double)(totcnt[jemx][e1]));
/*               sprintf(logger->tmptxt, "Single band source:  ee: %1d,  quality: %5.1lf\n", e1, quality);
   	       logger->logstat = logprint(logger->tmptxt, logger->lognum); 
*/
               if (quality > qual_limit) {
                  logger->tmptxt[0] = 0;
                  sprintf(logger->tmptxt2, "Single band strong source:  quality: %5.1lf  ", quality);
                  strcat(logger->tmptxt, logger->tmptxt2);
                  if (pointval > 0) {      
                     getRAdec(xx_g[jemx][e1][i], yy_g[jemx][e1][i], jemx, &s_RA, &s_dec, g_const, logger, 
                        jmx_id, backproj);
                     sprintf(logger->tmptxt2, "file: %s RA/dec: %+7.3f %+7.3f  ee: %1d  fit: %7.1lf ",
                              dfilnavn, s_RA, s_dec, e1, goodbet_g[jemx][e1][i]);
                     strcat(logger->tmptxt, logger->tmptxt2);
                     fprintf(srclistf, "%s  RA/dec: %+7.3f %+7.3f  ee: %1d  fit: %7.1lf",
                              dfilnavn,  s_RA, s_dec, e1, goodbet_g[jemx][e1][i]);
                  } else {
                     sprintf(logger->tmptxt2, "%s  XY: %+7.1f %+7.1f  strengths: %7.1lf ",
                              dfilnavn, avX0-(float)(backproj->FOV_radius), avY0-(float)(backproj->FOV_radius),
                              goodbet_g[jemx][e1][i]);
                     strcat(logger->tmptxt, logger->tmptxt2);
                     fprintf(srclistf, "%s  XY: %+7.1f %+7.1f  strengths: %7.1lf ",
                              dfilnavn, avX0-(float)(backproj->FOV_radius), avY0-(float)(backproj->FOV_radius), 
                              goodbet_g[jemx][e1][i]);
                  }
               
                  com_avx[k] = xx_g[jemx][e1][i]; 
                  com_avy[k] = yy_g[jemx][e1][i];
                  com_sn[k] = goodbet_g[jemx][e1][i] / sqrt((double)(backcnt[jemx][e1][i]));
                  backproj->com_RA[k] = s_RA;
                  backproj->com_dec[k] = s_dec;                  
                  k++;
                  fprintf(srclistf, "\n");
                  sprintf(logger->tmptxt2, "\n");
                  strcat(logger->tmptxt, logger->tmptxt2);
                  logger->logstat = logprint(logger->tmptxt, logger->lognum); 


                  accum0x[accu] = xx_g[jemx][e1][i]; 
                  accum0y[accu] = yy_g[jemx][e1][i]; 
                  accum1x[accu] = xx_g[jemx][e1][i]; 
                  accum1y[accu] = yy_g[jemx][e1][i];
                  a_orbit[accu] = backproj->aux_orbit;
                  a_pid[accu] = backproj->aux_pid;
                  a_pidv[accu] = aux_pidv;
                  accums0[accu] = goodbet_g[jemx][e1][i];
                  accums1[accu] = goodbet_g[jemx][e1][i];
                  actot0[accu] = backcnt[jemx][e1][i];
                  actot1[accu] = backcnt[jemx][e1][i];
                  accucode[accu] = jemx * 10000 + e1 * 100 + e1;
                  accumRA[accu] = s_RA;
                  accumdec[accu] = s_dec;
                  accumx[accu] = xx_g[jemx][e1][i]; 
                  accumy[accu] = yy_g[jemx][e1][i]; 
                  accums[accu] = goodbet_g[jemx][e1][i] / sqrt((double)(backcnt[jemx][e1][i]));
                  accumn0[accu] = mnum_g[jemx][e1][i]; 
                  accumn1[accu] = mnum_g[jemx][e1][i]; 
                  accu++;
               }
            }
         }
   }
                  
   n_common2 = k;
      
      /* ******************************************************** */
      /*                                                          */
      /*      redo source fit using only sources detected        */
      /*            in more than one energy band                  */
      /*                                                          */
      /* ******************************************************** */
      
      if (n_common2 > n_common) 
         sprintf(logger->tmptxt, 
            "\n Analysis using %d common and %d strong source(s) between 4 energy bands\n", 
            n_common, n_common2-n_common);
      else 
         sprintf(logger->tmptxt, 
            "\n Analysis using %d common source(s) between 4 energy bands\n", n_common);

      logger->logstat = logprint(logger->tmptxt, logger->lognum); 

      ksrc = backproj->backmodels + n_common2;

      for (i=0; i<n_common2; i++) {
         xx[backproj->backmodels+i] = com_avx[i];
         yy[backproj->backmodels+i] = com_avy[i];

         chk_no = 1;
         sprintf(logger->tmptxt, "source %1d:  X: %6.2f, Y: %6.2f\n", 
            i, xx[backproj->backmodels+i], yy[backproj->backmodels+i]);
         logger->logstat = logprint(logger->tmptxt, logger->lognum); 

      }
      
      n_factor = 100.0 / accumT[jemx];
      for (ee=0; ee<4; ee++) {
         
         if (n_common == 0) {
            sprintf(logger->tmptxt, "No common sources in the four energy bands\n"); 
            logger->logstat = logprint(logger->tmptxt, logger->lognum); 
            goto w_fits;
         }

         xrms = x_rms[ee];
         yrms = y_rms[ee];

         for (i=0; i<skydim; i++) imgpeak[ee][i] = 0.0;
      
         for (i=0; i<detdim; i++) {
            shadow[i] = shadow1[ee][i];
            backproj->soft_gr[i] = soft_gr1[ee][i];
         }
   
         jmx_id->absolen = abs_len[ee];
         jmx_id->scatt_len = sh_xy(xrms, yrms, logger, jmx_id); 
            /* calculate read-out scatter around X-ray absorbtion point at this energy */

         status = fun(-1, 1, ee, xx, yy, jemx, norm_d, 0, &funval, g_const, jmx_id, logger, backproj);
         /* reset fitting functions */
         if (status < 0) {
            logger->logstat = logprint(logger->error_text, logger->lognum); 
            return;
         }

         status = imagefit(shadow, jemx, ee, resid, ksrc, xx, yy, solut, norm_d, sumfun, chk_no, 
            &finalvar[ee], g_const, jmx_id, logger, backproj);
         if (status < 0) {
            logger->logstat = logprint(logger->error_text, logger->lognum); 
            return;
         }
            
         sh_rms[ee][3] = rmscalc(resid, backproj->soft_gr, logger);

         /* ************************************************************** */
         /*                                                                */
         /*    Backproject final residual shadowgram to form sky image     */
         /*                                                                */
         /* ************************************************************** */
      
         status = backpro_d(resid, backproj->skyim_r, logger, backproj);
         if (status < 0) {
            logger->logstat = logprint(logger->error_text, logger->lognum); 
            return;
         }

         clean2(backproj->skyim_r, xx, yy, n_common, backproj->backmodels, imgpeak[ee], logger, backproj);

         for (i=0; i<n_common2; i++) { 
            peakval[ee][i] = peaktransfer(xx[backproj->backmodels+i], yy[backproj->backmodels+i],
               			backproj->fitsimage[ee][2], imgpeak[ee], &peakrate, backproj, logger);
            peakval[ee+4][i] = peakrate * 100.0 * n_factor;
         }
         
         /* redo cleaning of raw image, now using the information about strong sources */
         clean2(backproj->fitsimage[ee][1], xx, yy, n_common2, backproj->backmodels, 
            		backproj->fitsimage[ee][2], logger, backproj);
         /* recover peaks of strong sources from 'imgpeak' image */
         for (i=0; i<n_common2; i++) { 
            peaktransfer(xx[backproj->backmodels+i], yy[backproj->backmodels+i], 
               imgpeak[ee], backproj->fitsimage[ee][2], &peakrate, backproj, logger);
         }
            
         if (f_ctrl == 2) {
            for (i=0; i<skydim; i++) {
               nn_factor = n_factor / (float)(backproj->illum[i]);
               backproj->fitsimage[ee][2][i]	*= nn_factor;
               imgpeak[ee][i]      		*= nn_factor;
               backproj->skyim_r[i]		*= nn_factor;
               backproj->fitsimage[ee][0][i]	*= nn_factor * nn_factor;
               if (backproj->fitsimage[ee][0][i] > 0.0) 
                  backproj->fitsimage[ee][3][i] = imgpeak[ee][i] / sqrt((double)(backproj->fitsimage[ee][0][i]));
               else backproj->fitsimage[ee][3][i] = 0.0;
                  
            }
         }
         
            /* after final fit output final, source cleaned map */
w_fits:             
         sprintf(fitsnavn, "skyima%1d_%1d_%04d_%03d%1d.fits", 
            jemx+1, ee, backproj->aux_orbit, backproj->aux_pid, aux_pidv);

         bitpix = -32;		/* use float representation in FITS file */
         dim1 = dim2 = backproj->FOV_radius * 2 + 1;	/* midisky image dimensions */
     
         cdelt = (jmx_id->mask_r + jmx_id->coll_r) * 180.0 / 
            	(backproj->FOV_radius * jmx_id->mask_height * g_const->pi);
/*         if (jemx == 0) jmxsign = 1; else jmxsign = -1;
*/
   	jmxsign = -1;
         rad_ang = p_posang[backproj->aux_pid] * g_const->pi / 180.0 + g_const->pi / 2.0 + jmx_id->phase2;
         sprintf(logger->tmptxt, 
            "At Funopen: cdelt: %10.8lf, rad_ang: %10.6lf, jmx_id->phase2: %8.6lf\n", cdelt, rad_ang, jmx_id->phase2);
         logger->logstat = logprint(logger->tmptxt, logger->lognum); 

         if (f_ctrl == 0) goto noFun;
         
         fi_headr(fitsnavn, logger);

         if( !(fun_handle = FunOpen(fitsnavn, "a", NULL)) ) 
            gerror(stderr, "could not FunOpen output file: %s\n", fitsnavn);
         else {
            /* prepare header */
            

            if (f_ctrl == 4)
               FunParamPuts(fun_handle, "EXTNAME", 0, "RAW", "initial intensity map - no flat fielding", 1);
            else 
               FunParamPuts(fun_handle, "EXTNAME", 0, "RAW_RECT", "Rectified initial intensity map", 1);
            FunParamPuts(fun_handle, "IMATYPE", 0, "INTENSITY", "Raw intensity map", 1);
            FunParamPuti(fun_handle, "VARIA_ID", 0, 100, "Variance map identifier", 1);
            FunParamPuts(fun_handle, "CTYPE1", 0, "RA---TAN", NULL, 1);
            FunParamPuts(fun_handle, "CTYPE2", 0, "DEC--TAN", NULL, 1);
            FunParamPuts(fun_handle, "CUNIT1", 0, "deg", NULL, 1);
            FunParamPuts(fun_handle, "CUNIT2", 0, "deg", NULL, 1);
            if (jemx == 1) {
               FunParamPuts(fun_handle, "INSTRUME", 0, "jmx2", NULL, 1);
               FunParamPuts(fun_handle, "VIGNDOL", 0, "NL_VIGNET2[1]", "Vignetting map name", 1);
            } else {
               FunParamPuts(fun_handle, "INSTRUME", 0, "jmx1", NULL, 1);
               FunParamPuts(fun_handle, "VIGNDOL", 0, "NL_VIGNET1[1]", "Vignetting map name", 1);
            }
            FunParamPutd(fun_handle, "CRVAL1", 0, p_RA[backproj->aux_pid], 12, NULL, 1);
            FunParamPutd(fun_handle, "CRVAL2", 0, p_dec[backproj->aux_pid], 12, NULL, 1);
            FunParamPutd(fun_handle, "ROT_ANG", 0, p_posang[backproj->aux_pid], 12, NULL, 1);
            FunParamPutd(fun_handle, "CRPIX1", 0, (double)((double)(backproj->FOV_radius+1) + 
               jmxsign*jmx_id->offY), 12, NULL, 1);
            FunParamPutd(fun_handle, "CRPIX2", 0, (double)((double)(backproj->FOV_radius+1) + 
               jmxsign*jmx_id->offX), 12, NULL, 1);
 
            FunParamPutd(fun_handle, "CD1_1", 0, cdelt * cos(rad_ang), 12, NULL, 1);
            FunParamPutd(fun_handle, "CD1_2", 0, cdelt * sin(rad_ang), 12, NULL, 1);
            FunParamPutd(fun_handle, "CD2_1", 0, cdelt * sin(rad_ang), 12, NULL, 1);
            FunParamPutd(fun_handle, "CD2_2", 0, -cdelt * cos(rad_ang), 12, NULL, 1);
 
            FunParamPutd(fun_handle, "EXPOSURE", 0, accumT[jemx], 12, "Effective integration time (s)", 1);
            FunParamPuti(fun_handle, "JMX_UNIT", 0, jemx+1, NULL, 1);
            FunParamPuti(fun_handle, "REVOL", 0, backproj->aux_orbit, NULL, 1);
            FunParamPuti(fun_handle, "PID", 0, backproj->aux_pid, NULL, 1);
            FunParamPuti(fun_handle, "PIDV", 0, aux_pidv, NULL, 1);
            FunParamPutd(fun_handle, "E_MIN", 0, eval[ee], 12, "keV", 1);
            FunParamPutd(fun_handle, "E_MAX", 0, eval[ee+1], 12, "keV", 1);
            FunParamPuts(fun_handle, "BKG_MODL", 0, ref_file_name, "Background model file", 1);

            if (f_ctrl == 4) {
               if( !FunImagePut(fun_handle, backproj->fitsimage[ee][4], dim1, dim2, bitpix, NULL) ) 
                  gerror(stderr, "could not FunImagePut: %s\n", fitsnavn);
            } else {
               if( !FunImagePut(fun_handle, backproj->fitsimage[ee][2], dim1, dim2, bitpix, NULL) ) 
                  gerror(stderr, "could not FunImagePut: %s\n", fitsnavn);
            }

            FunClose(fun_handle);
         }
         
         if( !(fun_handle = FunOpen(fitsnavn, "a", NULL)) ) 
            gerror(stderr, "could not FunOpen output file: %s\n", fitsnavn);
         else {

            FunParamPuts(fun_handle, "EXTNAME", 0, "VAR", "Variance map", 1);
            FunParamPuts(fun_handle, "IMATYPE", 0, "VARIANCE", "Variance map", 1);
            FunParamPuti(fun_handle, "VARIA_ID", 0, 100, "Variance map identifier", 1);
            FunParamPuts(fun_handle, "CTYPE1", 0, "RA---TAN", NULL, 1);
            FunParamPuts(fun_handle, "CTYPE2", 0, "DEC--TAN", NULL, 1);
            FunParamPuts(fun_handle, "CUNIT1", 0, "deg", NULL, 1);
            FunParamPuts(fun_handle, "CUNIT2", 0, "deg", NULL, 1);
            if (jemx == 1) {
               FunParamPuts(fun_handle, "INSTRUME", 0, "jmx2", NULL, 1);
               FunParamPuts(fun_handle, "VIGNDOL", 0, "NL_VIGNET2[1]", "Vignetting map name", 1);
            } else {
               FunParamPuts(fun_handle, "INSTRUME", 0, "jmx1", NULL, 1);
               FunParamPuts(fun_handle, "VIGNDOL", 0, "NL_VIGNET1[1]", "Vignetting map name", 1);
            }
            FunParamPutd(fun_handle, "CRVAL1", 0, p_RA[backproj->aux_pid], 12, NULL, 1);
            FunParamPutd(fun_handle, "CRVAL2", 0, p_dec[backproj->aux_pid], 12, NULL, 1);
            FunParamPutd(fun_handle, "ROT_ANG", 0, p_posang[backproj->aux_pid], 12, NULL, 1);
            FunParamPutd(fun_handle, "CRPIX1", 0, (double)((double)(backproj->FOV_radius+1) + 
               jmxsign*jmx_id->offY), 12, NULL, 1);
            FunParamPutd(fun_handle, "CRPIX2", 0, (double)((double)(backproj->FOV_radius+1) + 
               jmxsign*jmx_id->offX), 12, NULL, 1);

            FunParamPutd(fun_handle, "CD1_1", 0, cdelt * cos(rad_ang), 12, NULL, 1);
            FunParamPutd(fun_handle, "CD1_2", 0, cdelt * sin(rad_ang), 12, NULL, 1);
            FunParamPutd(fun_handle, "CD2_1", 0, cdelt * sin(rad_ang), 12, NULL, 1);
            FunParamPutd(fun_handle, "CD2_2", 0, -cdelt * cos(rad_ang), 12, NULL, 1);

            FunParamPutd(fun_handle, "EXPOSURE", 0, accumT[jemx], 12, "Effective integration time (s)", 1);
            FunParamPuti(fun_handle, "JMX_UNIT", 0, jemx+1, NULL, 1);
            FunParamPuti(fun_handle, "REVOL", 0, backproj->aux_orbit, NULL, 1);
            FunParamPuti(fun_handle, "PID", 0, backproj->aux_pid, NULL, 1);
            FunParamPuti(fun_handle, "PIDV", 0, aux_pidv, NULL, 1);
            FunParamPutd(fun_handle, "E_MIN", 0, eval[ee], 12, "keV", 1);
            FunParamPutd(fun_handle, "E_MAX", 0, eval[ee+1], 12, "keV", 1);
            FunParamPuts(fun_handle, "BKG_MODL", 0, ref_file_name, "Background model file", 1);

            if( !FunImagePut(fun_handle, backproj->fitsimage[ee][0], dim1, dim2, bitpix, NULL) ) 
               gerror(stderr, "could not FunImagePut: %s\n", fitsnavn);

            FunClose(fun_handle);
         }

         if (n_common2 == 0) continue;	/* goto next ee-value */
         
         if( !(fun_handle = FunOpen(fitsnavn, "a", NULL)) ) 
            gerror(stderr, "could not FunOpen output file: %s\n", fitsnavn);
         else {
            if (f_ctrl == 2) {
               FunParamPuts(fun_handle, "EXTNAME", 0, "SIG", "Significance map", 1);
               FunParamPuts(fun_handle,"IMATYPE", 0,"SIGNIFICANCE", "Significance map", 1);
            } else {
               FunParamPuts(fun_handle, "EXTNAME", 0, "RESIDUAL", "Residual intensity map - no flat fielding", 1);
               FunParamPuts(fun_handle, "IMATYPE", 0, "RESIDUAL", "Residual map (detected sources subtracted)", 1);
            }
            FunParamPuti(fun_handle, "VARIA_ID", 0, 100, "Variance map identifier", 1);
            FunParamPuts(fun_handle, "CTYPE1", 0, "RA---TAN", NULL, 1);
            FunParamPuts(fun_handle, "CTYPE2", 0, "DEC--TAN", NULL, 1);
            FunParamPuts(fun_handle, "CUNIT1", 0, "deg", NULL, 1);
            FunParamPuts(fun_handle, "CUNIT2", 0, "deg", NULL, 1);
            if (jemx == 1) {
               FunParamPuts(fun_handle, "INSTRUME", 0, "jmx2", NULL, 1);
               FunParamPuts(fun_handle, "VIGNDOL", 0, "NL_VIGNET2[1]", "Vignetting map name", 1);
            } else {
               FunParamPuts(fun_handle, "INSTRUME", 0, "jmx1", NULL, 1);
               FunParamPuts(fun_handle, "VIGNDOL", 0, "NL_VIGNET1[1]", "Vignetting map name", 1);
            }
            FunParamPutd(fun_handle, "CRVAL1", 0, p_RA[backproj->aux_pid], 12, NULL, 1);
            FunParamPutd(fun_handle, "CRVAL2", 0, p_dec[backproj->aux_pid], 12, NULL, 1);
            FunParamPutd(fun_handle, "ROT_ANG", 0, p_posang[backproj->aux_pid], 12, NULL, 1);
            FunParamPutd(fun_handle, "CRPIX1", 0, (double)((double)(backproj->FOV_radius+1) + 
               jmxsign*jmx_id->offY), 12, NULL, 1);
            FunParamPutd(fun_handle, "CRPIX2", 0, (double)((double)(backproj->FOV_radius+1) + 
               jmxsign*jmx_id->offX), 12, NULL, 1);

            FunParamPutd(fun_handle, "CD1_1", 0, cdelt * cos(rad_ang), 12, NULL, 1);
            FunParamPutd(fun_handle, "CD1_2", 0, cdelt * sin(rad_ang), 12, NULL, 1);
            FunParamPutd(fun_handle, "CD2_1", 0, cdelt * sin(rad_ang), 12, NULL, 1);
            FunParamPutd(fun_handle, "CD2_2", 0, -cdelt * cos(rad_ang), 12, NULL, 1);

            FunParamPutd(fun_handle, "EXPOSURE", 0, accumT[jemx], 12, "Effective integration time (s)", 1);
            FunParamPuti(fun_handle, "JMX_UNIT", 0, jemx+1, NULL, 1);
            FunParamPuti(fun_handle, "REVOL", 0, backproj->aux_orbit, NULL, 1);
            FunParamPuti(fun_handle, "PID", 0, backproj->aux_pid, NULL, 1);
            FunParamPuti(fun_handle, "PIDV", 0, aux_pidv, NULL, 1);
            FunParamPutd(fun_handle, "E_MIN", 0, eval[ee], 12, "keV", 1);
            FunParamPutd(fun_handle, "E_MAX", 0, eval[ee+1], 12, "keV", 1);
            FunParamPuti(fun_handle, "N_SRC", 0, n_common2, "Number of sources extracted", 1);
            FunParamPuts(fun_handle, "BKG_MODL", 0, ref_file_name, "Background model file", 1);

            if (f_ctrl == 2) {
               if( !FunImagePut(fun_handle, backproj->skyim_r, dim1, dim2, bitpix, NULL) ) 
                  gerror(stderr, "could not FunImagePut: %s\n", fitsnavn);
            } else {
               if( !FunImagePut(fun_handle, backproj->fitsimage[ee][3], dim1, dim2, bitpix, NULL) ) 
                  gerror(stderr, "could not FunImagePut: %s\n", fitsnavn);
            }

            FunClose(fun_handle);
         }

         if( !(fun_handle = FunOpen(fitsnavn, "a", NULL)) ) 
            gerror(stderr, "could not FunOpen output file: %s\n", fitsnavn);
         else {

            FunParamPuts(fun_handle, "EXTNAME", 0, "RES+SRC", "Residual intensity + sources", 1);
            FunParamPuts(fun_handle, "IMATYPE", 0, "RES+SRC", "Residual map + brushed up with sources", 1);
            FunParamPuti(fun_handle, "VARIA_ID", 0, 100, "Variance map identifier", 1);
            FunParamPuts(fun_handle, "CTYPE1", 0, "RA---TAN", NULL, 1);
            FunParamPuts(fun_handle, "CTYPE2", 0, "DEC--TAN", NULL, 1);
            FunParamPuts(fun_handle, "CUNIT1", 0, "deg", NULL, 1);
            FunParamPuts(fun_handle, "CUNIT2", 0, "deg", NULL, 1);
            if (jemx == 1) {
               FunParamPuts(fun_handle, "INSTRUME", 0, "jmx2", NULL, 1);
               FunParamPuts(fun_handle, "VIGNDOL", 0, "NL_VIGNET2[1]", "Vignetting map name", 1);
            } else {
               FunParamPuts(fun_handle, "INSTRUME", 0, "jmx1", NULL, 1);
               FunParamPuts(fun_handle, "VIGNDOL", 0, "NL_VIGNET1[1]", "Vignetting map name", 1);
            }
            FunParamPutd(fun_handle, "CRVAL1", 0, p_RA[backproj->aux_pid], 12, NULL, 1);
            FunParamPutd(fun_handle, "CRVAL2", 0, p_dec[backproj->aux_pid], 12, NULL, 1);
            FunParamPutd(fun_handle, "ROT_ANG", 0, p_posang[backproj->aux_pid], 12, NULL, 1);
            FunParamPutd(fun_handle, "CRPIX1", 0, (double)((double)(backproj->FOV_radius+1) + 
               jmxsign*jmx_id->offY), 12, NULL, 1);
            FunParamPutd(fun_handle, "CRPIX2", 0, (double)((double)(backproj->FOV_radius+1) + 
               jmxsign*jmx_id->offX), 12, NULL, 1);

            FunParamPutd(fun_handle, "CD1_1", 0, cdelt * cos(rad_ang), 12, NULL, 1);
            FunParamPutd(fun_handle, "CD1_2", 0, cdelt * sin(rad_ang), 12, NULL, 1);
            FunParamPutd(fun_handle, "CD2_1", 0, cdelt * sin(rad_ang), 12, NULL, 1);
            FunParamPutd(fun_handle, "CD2_2", 0, -cdelt * cos(rad_ang), 12, NULL, 1);

            FunParamPutd(fun_handle, "EXPOSURE", 0, accumT[jemx], 12, "Effective integration time (s)", 1);
            FunParamPuti(fun_handle, "JMX_UNIT", 0, jemx+1, NULL, 1);
            FunParamPuti(fun_handle, "REVOL", 0, backproj->aux_orbit, NULL, 1);
            FunParamPuti(fun_handle, "PID", 0, backproj->aux_pid, NULL, 1);
            FunParamPuti(fun_handle, "PIDV", 0, aux_pidv, NULL, 1);
            FunParamPutd(fun_handle, "E_MIN", 0, eval[ee], 12, "keV", 1);
            FunParamPutd(fun_handle, "E_MAX", 0, eval[ee+1], 12, "keV", 1);
            FunParamPuti(fun_handle, "N_SRC", 0, n_common2, "Number of sources extracted", 1);
            FunParamPuts(fun_handle, "COMMENT", 0, NULL, "This image contains the residual intensity", 1);
            FunParamPuts(fun_handle, "COMMENT", 0, NULL, "map with the raw source peaks restored", 1);
            FunParamPuts(fun_handle, "COMMENT", 0, NULL, "Only the pixels within the selected peak", 1); 
            FunParamPuts(fun_handle, "COMMENT", 0, NULL, "areas (5x5 pixels) have been transferred",1);
            FunParamPuts(fun_handle, "BKG_MODL", 0, ref_file_name, "Background model file", 1);

            if( !FunImagePut(fun_handle, imgpeak[ee], dim1, dim2, bitpix, NULL) ) 
               gerror(stderr, "could not FunImagePut: %s\n", fitsnavn);

            FunClose(fun_handle);
         }
noFun:
         if (j > 0) if (fabs(solut[0]) > solutold) goto loopstop4;

         solutold = fabs(solut[0]);
         rms_old = rms;

loopstop4:
         for (j=0; j<backproj->backmodels+n_common2; j++) {
            if (j >= backproj->backmodels) norm_d_g[jemx][ee][j] = 0.01 / (norm_d[j] * sumfun[j]); 
            else norm_d_g[jemx][ee][j] = 0.01 * backproj->soft_tot[26+ee];
            solut_e[ee][j] = solut[j];
         }
         
         if (norm_d_g[jemx][ee][0] > 0.0) {
            finalvar[ee] /= totcnt[jemx][ee] * 0.01 / norm_d_g[jemx][ee][0];	/* form reduced variance */
         
            sprintf(logger->tmptxt, 
               "E%1d shadowgram rel_sigma: raw: %10.3lf, pix_f: %10.3lf, backgr_fit: %10.3lf, src_fit: %10.3lf\n",
               ee, sh_rms[ee][0], sh_rms[ee][1], sh_rms[ee][2], sh_rms[ee][3]);
            logger->logstat = logprint(logger->tmptxt, logger->lognum); 
         }
/*
fprintf(srclist2, "E%1d shadowgram rel_sigma: raw: %10.3lf, pix_f: %10.3lf, backgr_fit: %10.3lf, src_fit: %10.3lf\n",
            ee, sh_rms[ee][0], sh_rms[ee][1], sh_rms[ee][2], sh_rms[ee][3]);
*/
         if (ee < 3) continue;
         
         dfilnavn[12] = 0;

         sprintf(resultxt, 
            "\n%s   Orbit: %3d, PID(scw): %3d %d, JEM-X%1d,  No of sources: %2d, Integr. time: %4.0lf s, ",
            dfilnavn, backproj->aux_orbit, backproj->aux_pid, aux_pidv, jemx+1, n_common2, accumT[jemx]);
         sprintf(dummy, "Pointing: RA/dec/R: %7.3lf %+7.3lf %6.2lf\n", 
            p_RA[backproj->aux_pid], p_dec[backproj->aux_pid], p_posang[backproj->aux_pid]);

         strcat(resultxt, dummy);
         sprintf(logger->tmptxt, "%s", resultxt);
         logger->logstat = logprint(logger->tmptxt, logger->lognum); 
         fprintf(srclist2, "%s", resultxt);

         sprintf(resultxt, 
            "Total cnts %5d %5d %5d %5d,   N_bckg: %4.2f %4.2f %4.2f %5.2f, ",
            totcnt[jemx][0], totcnt[jemx][1], totcnt[jemx][2], totcnt[jemx][3],  
            (float)(solut_e[0][0] / accumT[jemx]), (float)(solut_e[1][0] / accumT[jemx]), 
            (float)(solut_e[2][0] / accumT[jemx]), (float)(solut_e[3][0] / accumT[jemx])); 
         
         sprintf(dummy, " E_bckg: %5.2f %5.2f %5.2f %5.2f\n",
            (float)(solut_e[0][1] / accumT[jemx]), (float)(solut_e[1][1] / accumT[jemx]), 
            (float)(solut_e[2][1] / accumT[jemx]), (float)(solut_e[3][1] / accumT[jemx])); 

         strcat(resultxt, dummy);

         sprintf(logger->tmptxt, "%s", resultxt);
         logger->logstat = logprint(logger->tmptxt, logger->lognum); 
         fprintf(srclist2, "%s", resultxt);

         maxsss = 0.0;
         maxjb = -1;
         for (j=0; j<n_common2; j++) {
            jb = backproj->backmodels + j;
            if ((abs(backproj->com_RA[j] - 83.6) < 0.2) && (abs(backproj->com_dec[j] - 22.0) < 0.2)) {
               sss = solut_e[0][jb] + solut_e[1][jb] + solut_e[2][jb] + solut_e[3][jb];
               if (sss > maxsss) {
                  maxsss = sss;
                  maxjb = jb;
               }
            }
         }

         for (j=0; j<n_common2; j++) {
            jb = backproj->backmodels + j;
            
            cmx = com_avx[j] - backproj->FOV_radius;
            cmy = com_avy[j] - backproj->FOV_radius;
            avr = sqrt(cmx*cmx + cmy*cmy);
            
            sprintf(resultxt, "x/y/r: %6.2f %6.2f %6.2f, RA/dec: %6.2f %+6.2f, SN: %6.2f, ",
            	cmx, cmy, avr, backproj->com_RA[j], backproj->com_dec[j], com_sn[j]);

            sprintf(dummy, "Real/Norm cps: %6.3f %6.2f  %6.3f %6.2f  %6.3f %6.2f  %6.3f %6.2f\n",
                (float)(solut_e[0][jb] / accumT[jemx]), 
            	(float)(solut_e[0][jb] * 100.0 / (accumT[jemx] * norm_d_g[jemx][0][jb])), 
                (float)(solut_e[1][jb] / accumT[jemx]), 
                (float)(solut_e[1][jb] * 100.0 / (accumT[jemx] * norm_d_g[jemx][1][jb])), 
                (float)(solut_e[2][jb] / accumT[jemx]), 
                (float)(solut_e[2][jb] * 100.0 / (accumT[jemx] * norm_d_g[jemx][2][jb])), 
                (float)(solut_e[3][jb] / accumT[jemx]), 
                (float)(solut_e[3][jb] * 100.0 / (accumT[jemx] * norm_d_g[jemx][3][jb])));

            strcat(resultxt, dummy);
            sprintf(logger->tmptxt, "%s", resultxt);               
            logger->logstat = logprint(logger->tmptxt, logger->lognum); 
            fprintf(srclist2, "%s", resultxt);
            fprintf(srclist2,
               "Image peak cnts: %6.2f %6.2f %6.2f %6.2f  cnts/cm2/s: %6.2f %6.2f %6.2f %6.2f\n",
               peakval[0][j], peakval[1][j], peakval[2][j], peakval[3][j], 
               peakval[4][j], peakval[5][j], peakval[6][j], peakval[7][j]);

   	    if (jb == maxjb) {
               gnudat[gnunum][0][0] = avr * 6.6 / (float)(backproj->FOV_radius);
               gnudat[gnunum][0][1] = (float)(solut_e[0][jb] * 100.0 / (accumT[jemx] * norm_d_g[jemx][0][jb]));
               gnudat[gnunum][0][2] = peakval[4][j];               
               gnudat[gnunum][1][0] = avr * 6.6 / (float)(backproj->FOV_radius);
               gnudat[gnunum][1][1] = (float)(solut_e[1][jb] * 100.0 / (accumT[jemx] * norm_d_g[jemx][1][jb]));
               gnudat[gnunum][1][2] = peakval[5][j];               
               gnudat[gnunum][2][0] = avr * 6.6 / (float)(backproj->FOV_radius);
               gnudat[gnunum][2][1] = (float)(solut_e[2][jb] * 100.0 / (accumT[jemx] * norm_d_g[jemx][2][jb]));
               gnudat[gnunum][2][2] = peakval[6][j];               
               gnudat[gnunum][3][0] = avr * 6.6 / (float)(backproj->FOV_radius);
               gnudat[gnunum][3][1] = (float)(solut_e[3][jb] * 100.0 / (accumT[jemx] * norm_d_g[jemx][3][jb]));
               gnudat[gnunum][3][2] = peakval[7][j];
               gnunum++;
               if (gnunum > 199) gnunum = 199;
            }
         }
      }

      goto next_file;
final:
      fclose(fillist);

      for (i=0; i<m_num; i++) {
         used2[i] = 0;
      }
      sprintf(logger->tmptxt, "Total number of source candidates found: %5d:\n", m_num);
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
         
      accu2 = 0;
      accu3 = 0;
      for (i=0; i<m_num-1; i++) {
         if ((used2[i] > 0) || (m_lst_s[i] < 0.0)) continue;
         sumX = m_lst_ra[i] * m_lst_s[i];
         sumY = m_lst_dec[i] * m_lst_s[i];
         sumgood = sumsn = m_lst_s[i];
         sumcnts = totsn = m_lst_cnt[i];
         sn_best = m_lst_s[i] / sqrt((double)(m_lst_cnt[i]));
         cosdec = cos((double)(m_lst_dec[i]) * g_const->gtorad);
         sumtot = 1;
         pidlast = m_lst_pid[i];
         e_indic = 1 << (m_lst_e[i] * 7);
                  
         for (j=i+1; j<m_num; j++) {
            if ((used2[j] > 0) || (m_lst_s[j] < 0.0)) continue;
            if ((m_lst_pid[i] == m_lst_pid[j]) && (m_lst_e[i] == m_lst_e[j])) continue;
            
            if (sqrt((double)((m_lst_ra[i] -m_lst_ra[j] ) * (m_lst_ra[i] -m_lst_ra[j] ))*cosdec*cosdec + 
                     (double)((m_lst_dec[i]-m_lst_dec[j]) * (m_lst_dec[i]-m_lst_dec[j]))) < 0.05) { 

               if (used1[i] == 0) {      
                  accu2RA[accu2] = m_lst_ra[i];
                  accu2dec[accu2] = m_lst_dec[i];
                  accum2x[accu2] = m_lst_x[i];
                  accum2y[accu2] = m_lst_y[i];
                  acc2pid[accu2] = m_lst_pid[i];
                  acc2sn[accu2] =  sqrt((double)(m_lst_s[i] / m_lst_cnt[i]));
                  accums2[accu2] =  sqrt((double)(m_lst_s[i]));
                  actot2[accu2] =  m_lst_cnt[i];
                  acc2ee[accu2] = m_lst_e[i];
                  acc2jx[accu2] = jemx;
                  accu2++;
                  if (accu2 >= accumax) accu2 = accumax-1;
               }
               accu2RA[accu2] = m_lst_ra[j];
               accu2dec[accu2] = m_lst_dec[j];
               accum2x[accu2] = m_lst_x[j];
               accum2y[accu2] = m_lst_y[j];
               acc2pid[accu2] = m_lst_pid[j];
               acc2sn[accu2] =  sqrt((double)(m_lst_s[j] / m_lst_cnt[j]));
               accums2[accu2] =  sqrt((double)(m_lst_s[j]));
               actot2[accu2] =  m_lst_cnt[j];
               acc2ee[accu2] = m_lst_e[j];
               acc2jx[accu2] = jemx;
               accu2++;
               if (accu2 >= accumax) accu2 = accumax-1;

               sumX += m_lst_ra[j] * m_lst_s[j];
               sumY += m_lst_dec[j] * m_lst_s[j];

               tempgood = sumsn + m_lst_s[j];
               temptot = totsn + m_lst_cnt[j];
               sn_temp = tempgood / sqrt((double)(temptot));
               sn_new = m_lst_s[j] / sqrt((double)(m_lst_cnt[j]));

               if ((sn_temp >= sn_best) && (sn_temp >= sn_new )) {
                  sn_best = sn_temp;
                  sumsn = tempgood;
                  totsn = temptot;
               }
               if ((sn_new  >= sn_best) && (sn_new  >= sn_temp)) {
                  sn_best = sn_new;
                  sumsn = goodbet_g[jemx][e2][j];
                  totsn = totcnt[jemx][e2];
               }
                        
               sumgood += m_lst_s[j];
               sumcnts += m_lst_cnt[j];
               sumtot++;

               e_indic += 1 << (m_lst_e[j] * 7);
               
               if (pidlast != m_lst_pid[j]) sumtot += 1000; 
               pidlast = m_lst_pid[j];
                                    
               avX0 = sumX / sumgood;
               avY0 = sumY / sumgood;
               used1[i] = 1;
               used2[j] = 1;
            }
            if (used1[i] == 1) {      
               if (i == last_i) continue;
               last_i = i;
               accum3x[accu3] = avX0;
               accum3y[accu3] = avY0;
               acc3tot[accu3] = totsn;
               acc3sn[accu3] = sn_best;
               acc3ee[accu3] = e_indic;
               accu3++;
               if (accu3 >= accumax) accu3 = accumax-1;
            }
         }
      }   


   fprintf(cat_fil, "\n\n");
   for (i=0; i<accu; i++) {
      j = accucode[i] / 10000;
      k = accucode[i] % 100;
      fprintf(cat_fil, "1  %3d %4d %4d %3d %3d %3d %7.3f %7.3f %10.1f %10.1f %7.3f %7.3f \n", 
      	i, a_orbit[i], a_pid[i], a_pidv[i], j, k, accum0x[i], accum0y[i], 
        accums0[i], actot0[i], accumRA[i], accumdec[i]);
   }
   fprintf(cat_fil, "\n\n");
   for (i=0; i<accu; i++) {
      j = accucode[i] / 10000;
      k = ((accucode[i] - j * 10000) / 100) % 100;
      fprintf(cat_fil, "2  %3d %4d %4d %3d %3d %3d %7.3f %7.3f %10.1f %10.1f %7.3f %7.3f \n", 
        i, a_orbit[i], a_pid[i], a_pidv[i], j, k, accum1x[i], accum1y[i], 
        accums1[i], actot1[i], accumRA[i], accumdec[i]);
   }
   fprintf(cat_fil, "\n\n");
   
   for (i=0; i<accu3; i++) {
      ee0 = acc3ee[i] % 128;
      ee1 = (acc3ee[i] >> 7) % 128;
      ee2 = (acc3ee[i] >> 14) % 128;
      ee3 = (acc3ee[i] >> 21) % 128;
      fprintf(cat_fil, "3  %3d  %7.3f  %7.3f  %12.1f  %8d   %3d %3d %3d %3d\n", 
      i, accum3x[i], accum3y[i], acc3sn[i], acc3tot[i], ee0, ee1, ee2, ee3);
   }
   fprintf(cat_fil, "\n\n");

   for (i=0; i<accu2; i++) {
      orbit = acc2pid[i] / 100000;
      pid  = (acc2pid[i] / 100) % 1000;
      pidv  = acc2pid[i] % 100;

      fprintf(cat_fil, "4  %3d %4d %4d %3d %3d %3d", i, orbit, pid, pidv, acc2jx[i]+1, acc2ee[i]);
      fprintf(cat_fil, " %7.3f %7.3f %10.1f %10.1f", accum2x[i], accum2y[i], accums2[i], actot2[i]);
      fprintf(cat_fil, " %7.3lf %7.3lf %7.3lf %7.3f %7.3f\n", 
         		p_RA[pid], p_dec[pid], p_posang[pid], accu2RA[i], accu2dec[i]);
   }
   for (i=0; i<accu2; i++) {
   
      if (acc2sn[i] < 4.5) continue;
      
      orbit = acc2pid[i] / 100000;
      pid  = (acc2pid[i] / 100) % 1000;
      pidv  = acc2pid[i] % 100;

      getxy(accu2RA[i], accu2dec[i], jemx, &x, &y, g_const, jmx_id, backproj, logger);

      printf("foundXY %7.3f %7.3f %10.1f %10.1f, calc XY from RA/dec: %7.3f %7.3f\n", 
         	accum2x[i], accum2y[i], accums2[i], actot2[i], x, y);
      fprintf(cat_fil, "5  foundXY %7.3f %7.3f %10.1f %10.1f, calc XY from RA/dec: %7.3f %7.3f\n", 
         	accum2x[i], accum2y[i], accums2[i], actot2[i], x, y);

/*      d_dec_GX340 = (accu2dec[i] - dec_GX340);
      d_RA_GX340 = (accu2RA[i] - RA_GX340) * cos(dec_GX340);
      dist_GX340 = sqrt(d_dec_GX340 * d_dec_GX340 + d_RA_GX340 * d_RA_GX340);

      d_dec_GX337 = (accu2dec[i] - dec_GX337);
      d_RA_GX337 = (accu2RA[i] - RA_GX337) * cos(dec_GX337);
      dist_GX337 = sqrt(d_dec_GX337 * d_dec_GX337 + d_RA_GX337 * d_RA_GX337);
      
      if (dist_GX340 <= dist_GX337) {
         dd_dec = d_dec_GX340 * 120.0;*/	/* units: 30 arcseconds */
/*         dd_RA = d_RA_GX340 * 120.0;
         dd_dist = dist_GX340 * 200.0;
         dd_dec += 10.0;
         dd_RA += 10.0;
         if (dd_dec < 0.0) dd_dec = 0.0;
         if (dd_RA < 0.0) dd_RA = 0.0;
         if (dd_dec > 19.0) dd_dec = 19.0;
         if (dd_RA > 19.0) dd_RA = 19.0;
         if (dd_dist > 29.0) dd_dist = 29.0;
         arr_dec_GX340[(int)(dd_dec)]++;
         arr_RA_GX340[(int)(dd_RA)]++;
         if (acc2sn[i] < 14.0) {
            arr_dist_GX340[(int)(dd_dist)]++;
            arr_sn_GX340[(int)(acc2sn[i])]++;
         } else {
            arr2_dist_GX340[(int)(dd_dist)]++;
            if (acc2sn[i] < 29.0) arr2_sn_GX340[(int)(acc2sn[i])]++; else arr2_sn_GX340[29]++;
         }
      } else {
         dd_dec = d_dec_GX337 * 120.0;
         dd_RA = d_RA_GX337 * 120.0;
         dd_dist = dist_GX337 * 200.0;
         dd_dec += 10.0;
         dd_RA += 10.0;
         if (dd_dec < 0.0) dd_dec = 0.0;
         if (dd_RA < 0.0) dd_RA = 0.0;
         if (dd_dec > 19.0) dd_dec = 19.0;
         if (dd_RA > 19.0) dd_RA = 19.0;
         if (dd_dist > 29.0) dd_dist = 29.0;
         arr_dec_GX337[(int)(dd_dec)]++;
         arr_RA_GX337[(int)(dd_RA)]++;
         arr_dist_GX337[(int)(dd_dist)]++;
         if (acc2sn[i] < 29.0) arr_sn_GX337[(int)(acc2sn[i])]++; else arr_sn_GX337[29]++;
      }
*/
   }
/*   fprintf(cat_fil, "\n6  GX340 RA:   ");
   for (i=0; i<20; i++) fprintf(cat_fil, " %3d", arr_RA_GX340[i]); 
   fprintf(cat_fil, "\n6  GX340 dec:  ");
   for (i=0; i<20; i++) fprintf(cat_fil, " %3d", arr_dec_GX340[i]); 
   fprintf(cat_fil, "\n6  GX340 dist: ");
   for (i=0; i<30; i++) fprintf(cat_fil, " %3d", arr_dist_GX340[i]); 
   fprintf(cat_fil, "\n6  GX340 S/N:  ");
   for (i=0; i<30; i++) fprintf(cat_fil, " %3d", arr_sn_GX340[i]); 
   fprintf(cat_fil, "\n6  GX340 dist: ");
   for (i=0; i<30; i++) fprintf(cat_fil, " %3d", arr2_dist_GX340[i]); 
   fprintf(cat_fil, "\n6  GX340 S/N:  ");
   for (i=0; i<30; i++) fprintf(cat_fil, " %3d", arr2_sn_GX340[i]); 
   fprintf(cat_fil, "\n\n6  GX337 RA:   ");
   for (i=0; i<20; i++) fprintf(cat_fil, " %3d", arr_RA_GX337[i]); 
   fprintf(cat_fil, "\n6  GX337 dec:  ");
   for (i=0; i<20; i++) fprintf(cat_fil, " %3d", arr_dec_GX337[i]); 
   fprintf(cat_fil, "\n6  GX337 dist: ");
   for (i=0; i<30; i++) fprintf(cat_fil, " %3d", arr_dist_GX337[i]); 
   fprintf(cat_fil, "\n6  GX337 S/N:  ");
   for (i=0; i<30; i++) fprintf(cat_fil, " %3d", arr_sn_GX337[i]); 
*/
   fprintf(cat_fil, "\n\n");

   sprintf(logger->tmptxt, "  Accepted and rejected shadowgram counts:\n");
   logger->logstat = logprint(logger->tmptxt, logger->lognum); 
   sprintf(logger->tmptxt, "E0:  %8d    %8d\n", s_accept[0], s_reject[0]);
   logger->logstat = logprint(logger->tmptxt, logger->lognum); 
   sprintf(logger->tmptxt, "E1:  %8d    %8d\n", s_accept[1], s_reject[1]);
   logger->logstat = logprint(logger->tmptxt, logger->lognum); 
   sprintf(logger->tmptxt, "E2:  %8d    %8d\n", s_accept[2], s_reject[2]);
   logger->logstat = logprint(logger->tmptxt, logger->lognum); 
   sprintf(logger->tmptxt, "E3:  %8d    %8d\n", s_accept[3], s_reject[3]);
   logger->logstat = logprint(logger->tmptxt, logger->lognum); 

   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   rms = 0.0;
   for (i=0; i<logger->func_used_num; i++) {
      logger->func_times[i] /= CLOCKS_PER_SEC;
      printf("%3d %15s: calls: %8d, total time: %8.3lf s\n", 
         i, logger->func_names[i], logger->func_calls[i], logger->func_times[i]);
      fprintf(cat_fil, "0  %3d %15s: calls: %8d, total time: %8.3lf s\n", 
         i, logger->func_names[i], logger->func_calls[i], logger->func_times[i]);

      rms += logger->func_times[i];

   }
   printf("Total time used: %9.1lf s\n", rms);
   fprintf(cat_fil, "0  Total time used: %9.1lf s\n", rms);
   fprintf(cat_fil, "\n\n");

   fclose(cat_fil);
   fclose(srclistf);
   fclose(srclist2);

   if (gnunum == 0) return;
   
   cat_fil = fopen("md8.gnu", "wt");
   for (ee=0; ee<4; ee++) {
      for (i=0; i<gnunum; i++) fprintf(cat_fil, "%6.3f  %8.2f\n", gnudat[i][ee][0], gnudat[i][ee][1]);
      fprintf(cat_fil, "\n\n");
      for (i=0; i<gnunum; i++) fprintf(cat_fil, "%6.3f  %8.2f\n", gnudat[i][ee][0], gnudat[i][ee][2]);
      fprintf(cat_fil, "\n\n");
   }
   for (i=0; i<gnunum; i++) fprintf(cat_fil, "%6.3f  %8.2f\n", gnudat[i][ee][0], 
      gnudat[i][0][1] + gnudat[i][1][1] + gnudat[i][2][1] + gnudat[i][3][1]);
   fprintf(cat_fil, "\n\n");
   for (i=0; i<gnunum; i++) fprintf(cat_fil, "%6.3f  %8.2f\n", gnudat[i][ee][0], 
      gnudat[i][0][2] + gnudat[i][1][2] + gnudat[i][2][2] + gnudat[i][3][2]);
   fprintf(cat_fil, "\n\n");
   fclose(cat_fil);
   
   return;
}

/* ******************************************************************** */

int backsky_init(int jemx, int dead_ano, char sky_version, int FOV_ff, 
   	char* full_ref_name, struct log_con *logger, struct backpro *backproj, 
        struct instr_data *jmx_id, struct euclid *g_const)	
   /* initialize global parameters */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                             */        
/*                                                             */        
/*                                                             */
/*  Niels Lund, February 2003                                  */        
/*                                                             */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

{
   FILE *sp_fil, *datafil, *ifil, *pofil;
   int i, j, k, m, n, bytes, indent = 1, minsum, x, y, r2, pixindex, FOV2, status;
   char phline[128], skymapa[128], skymap[128], skyfil[200], irosfil[200], pixfil[200], skofil[200], srcnam[128];
   char illumline[2000], detmapfil[200];
   static int done_data_input = 0;
   double softsum, difsum=0;
   char dummy[100], pointfil[200];
   short maskhole[22501];		/* 22501 is the "holy" prime number on which the JEM-X mask is based */ 

   static int func_local = -1, local_calling_num=0;
   clock_t clock(void), calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "backsky_init");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
      /* ++++++++++++++ The following part is only executed once ! +++++++++++++++++ */
      
   if (done_data_input == 1) goto skip; 
    
   done_data_input = 1;
   
   sprintf(skyfil, "%s/reference/skmap%1c%1dc%1d_a", 
      logger->sweethome, sky_version, FOV_ff, jemx+1);			/* default backprojection file */
   sprintf(logger->tmptxt, "Default backprojection file: %s\n", skyfil);
   logger->logstat = logprint(logger->tmptxt, logger->lognum); 
   fprintf(cat_fil, "0  backprojection file: %s\n", skyfil);
      
   status = backpro_init(skyfil, parameters, logger);
   
   backproj->indep = parameters[10];
   
   if (dead_ano == 1) 
      sprintf(skofil, "%s/reference/period5new/smooth%1d+anod.bin", logger->sweethome, jemx+1);
   else sprintf(skofil, "%s/reference/period5new/smooth%1d-anod.bin", logger->sweethome, jemx+1);

   datafil = fopen(skofil, "rb");
   if (datafil == NULL) {
      sprintf(logger->tmptxt, "\nCould not open %s, goodbye\n", skofil);
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      return(-1);
   }
   
   fprintf(cat_fil, "0 background reference shadowgram read from: %s\n", skofil);

   fread(soft_shdgr, 4, 4*detdim, datafil);
   fread(backproj->soft_tot, 8, 30, datafil);
   fclose(datafil);
   
/*   for (i=0; i<30; i++) {
      sprintf(logger->tmptxt, "soft_tot[%2d]: %10.4lf\n", i, backproj->soft_tot[i]);
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
   }   
*/

   sprintf(pixfil, "%s/reference/pixdata%1d.bin", logger->sweethome, jemx+1);
   fprintf(cat_fil, "0  pixel file: %s\n", pixfil);
   
   ifil = fopen(pixfil, "rb");
   if (ifil == NULL) {
      sprintf(logger->tmptxt, " Error in opening %s\n", pixfil); 
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      return(-1);
   }
   fread(jmx_id->pixel_st, 4, detdim, ifil);
   pixindex = jmx_id->pixel_st[0];
   fread(jmx_id->pixel_list, 1, pixindex, ifil);
   fclose(ifil);

   softsum = 0.0;
   for (i=1; i<22501; i++) {
      softsum += maskhole[i];
   }
        
   backproj->FOV_radius = -1;

   status = alignment_init(jemx, FOV_ff, logger, jmx_id, backproj, g_const);

   if (status < 0) {
      sprintf(logger->tmptxt, "Bad or missing alignment.dat, exiting!\n"); 
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      return(-1);
   }
   if (backproj->FOV_radius != FOV_rad) {
      sprintf(logger->tmptxt, "FOV_radius not matching in alignment data\n"); 
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      return(-1);
   }

   backproj->depth_corr = jmx_id->mask_height / (jmx_id->mask_height + backproj->indep);
   
      /* dead_ano = 0;  default: reject bad anodes */

   if (dead_ano == 0) sprintf(detmapfil, "%s/reference/detmap%1c%1d-%1d.aux", logger->sweethome, sky_version, FOV_ff, jemx+1);
   else sprintf(detmapfil, "%s/reference/detmap%1c%1d+%1d.aux", logger->sweethome, sky_version, FOV_ff, jemx+1);
   
   datafil = fopen(detmapfil, "rb");
   if (datafil == NULL) {
      sprintf(logger->tmptxt, "\nCould not open %s, goodbye\n", detmapfil);
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      return(-1);
   }
   fread(backproj->illum, 4, skydim, datafil);
   fclose(datafil);

   m = n = 0;
   FOV2 = backproj->FOV_radius * backproj->FOV_radius;
   for (j=1; j<skydim; j++) {
      if (backproj->illum[j] == 0) continue;
      if (backproj->illum[j] > n) n = backproj->illum[j];
      backproj->rsignif[j] = 1.0 / sqrt((double)(backproj->illum[j]));
      if (backproj->illum[j] > backproj->signiflim) backproj->access[j] = 1; else backproj->access[j] = 0;
      x = (float)(j / sky_ydim) - (float)(backproj->FOV_radius);
      y = (float)(j % sky_ydim) - (float)(backproj->FOV_radius);
      if ((x*x + y*y) > FOV2) backproj->access[j] = 0;
      m++;
      k = backproj->illum[j] / 50;
      if (k > 29) k = 29;
   }
   for (i=0; i<32; i++) bb[i] = 1<<i;

   sprintf(srcnam, "n_sources.txt");
   srclistf = fopen(srcnam,"wt") ;
   if ( srclistf == NULL) {
      sprintf(logger->tmptxt, "Error opening file: %s ..... exiting!\n", srcnam);
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      return(-1);
   }
   
   sprintf(srcnam, "n_sources2.txt");
   srclist2 = fopen(srcnam,"wt") ;
   if ( srclist2 == NULL) {
      sprintf(logger->tmptxt, "Error opening file: %s ..... exiting!\n", srcnam);
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      return(-1);
   }
   
   datafil = fopen(full_ref_name, "rb");
   if (datafil == NULL) {
      sprintf(logger->tmptxt, "\nCould not open %s, goodbye\n", full_ref_name);
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      return(-1);
   }
   fread(backproj->empty, 4, 4*detdim, datafil);
   fread(backproj->empty_aux, 8, 30, datafil);
   fclose(datafil);
   
   for (i=0; i<30; i+=3) {
      sprintf(logger->tmptxt, "empty_aux[%2d/%2d/%2d]: %12.3lf,  %12.3lf,  %12.3lf\n", 
         i, i+1, i+2, backproj->empty_aux[i], backproj->empty_aux[i+1], backproj->empty_aux[i+2]);
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
   }

skip:
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return(0);
}

/* ******************************************************************** */

int pixelfold(short det[], int ee, float shadw[detdim], struct log_con *logger, struct backpro *backproj,
   	struct instr_data *jmx_id)
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                             */        
/*  pixelfold.c corrects for the nonuniform readout grid of    */
/*  real microstrid detector. The counts recorded in a         */
/*  particular telemetry pixel is distributed among idealized  */
/*  1x1 mm2 pixels on an idealized microstrip detector         */        
/*                                                             */
/*  Niels Lund, February 2003                                  */        
/*                                                             */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

{
   int i, j, k, m, n, len, index, delta, val, valnum;
   double sum1, sum2, loss1, valsum, vals;

   static int func_local = -1, local_calling_num=0;
   clock_t calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "pixelfold");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
   for (i=0; i<detdim; i++) shadw[i] = 0.0;
   valsum = vals = 0.0;
   valnum = 0;
   
   sum1 = loss1 = 0.0;
   for (i=1; i<detdim; i++) {
      if ((det[i] <= 0) || (backproj->soft_gr[i] <= 0.0)) continue;
      sum1 += det[i];
      vals = 0.0;
      index = jmx_id->pixel_st[i];
      len = jmx_id->pixel_list[index++];
      for (j=0; j<len; j++) {
         delta = jmx_id->pixel_list[index++];
         val = jmx_id->pixel_list[index++];
         vals += val;
         k = ((delta & 15) - 8); 
         m = ((delta & 240) - 128) * 16;
         n = i + k + m;
         if ((n < 0) || (n >= detdim)) {
            loss1 += val * (float)(det[i]);
         } else {
            shadw[n] += val * (float)(det[i]);         
         }
      }
      valsum += vals / 255.0;
      valnum++;
      if ((vals > 260.0) || (vals < 250.0)) {
         index = jmx_id->pixel_st[i];
         len = jmx_id->pixel_list[index++];
         sprintf(logger->tmptxt, "%6d pixel: len: %1d, [delta/delta'/val]: ", i, len); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum); 

         n = 0;
         for (j=0; j<len; j++) {
            delta = jmx_id->pixel_list[index++];
            val = jmx_id->pixel_list[index++];
            n += val;
            k = ((delta & 15) - 8); 
            m = ((delta & 240) - 128) * 16;
            m += k;
 	    sprintf(logger->tmptxt, " %2x/%+4d/%3d  ", delta, m, val); 
            logger->logstat = logprint(logger->tmptxt, logger->lognum); 

         }
 	 sprintf(logger->tmptxt, " sum: %3d\n", n); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum); 

         if ((vals > 260.0) || (vals < 250.0)) exit(0);
      }
   }
   
   sum2 = 0.0;
   for (i=0; i<detdim; i++) {
      if (backproj->soft_gr[i] < 0.0) shadw[i] = 0;
      shadw[i] /= 255.0;
      sum2 += shadw[i];
   }
   loss1 /= 255.0;
/*   for (i=0; i<detdim; i++) shadw[i] /= sum2;
*/
   sprintf(logger->tmptxt, 
     "pixelfold: sum_in: %10.4lf,     loss: %10.4lf,    sum_out: %10.4lf, average valsum: %6.4lf\n", 
      sum1, loss1, sum2, valsum/valnum);
   logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return((int)(sum2));
}

/* *********************************************************** */

double rmscalc(float shad[], float soft_gr[], struct log_con *logger)
{
   int i, j;
   double m0, m1, m2, rms, var;
   
   static int func_local = -1, local_calling_num=0;
   clock_t calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "rmscalc");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
   m0 = m1 = m2 = 0.0;
   
   for (i=0; i<detdim; i++) {
      if (soft_gr[i] <= 0.0) continue;
      m0 += 1.0;
      m1 += shad[i];
      m2 += shad[i] * shad[i];
   }
   
   m1 /= m0;
   m2 /= m0;

   var = m2 - m1 * m1;
   if (var > 0.0) rms = sqrt(var);
   else rms = 0.0;
   
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return(rms);
}

/* *********************************************************** */

double rmscalc_i(short shad[], float soft_gr[], struct log_con *logger)
{
   int i, j;
   double m0, m1, m2, rms, var;
   
   static int func_local = -1, local_calling_num=0;
   clock_t calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "rmscalc_i");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
   m0 = m1 = m2 = 0.0;
   
   for (i=0; i<detdim; i++) {
      if (soft_gr[i] <= 0.0) continue;
      m0 += 1.0;
      m1 += (double)(shad[i]);
      m2 += (double)(shad[i]) *(double)(shad[i]);
   }
   
   m1 /= m0;
   m2 /= m0;

   var = m2 - m1 * m1;
   if (var > 0.0) rms = sqrt(var);
   else rms = 0.0;
   
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return(rms);
}

/* ********************************************************** */

void clean(float sky_r[skydim], float skyim[skydim], struct log_con *logger)
{
   int i, j, n, k, m, max_j[6] = {-1, -1, -1, -1, -1, -1};
   int sky_h, min_j[6];
   float b, v, max[6] = {-100000.0, -100000.0, -100000.0, -100000.0, -100000.0, -100000.0};
   float min[6];
   static int first = 1, indx[20];

   static int func_local = -1, local_calling_num=0;
   clock_t calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "clean");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
      
   if (first == 1) {
      first = 0; 
      m = 0;
      for (i=0; i<5; i++) {
         indx[m++] = (5-i)*sky_ydim + i; 
         indx[m++] = -i*sky_ydim + (5-i); 
         indx[m++] = -(5-i)*sky_ydim - i;
         indx[m++] = i*sky_ydim - (5-i);
      }
/*      for (i=0; i<m; i++) {
   	 sprintf(logger->tmptxt, "%2d  %5d\n", i, indx[i]);
         logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      }
*/
   }
   sky_h = (skydim - 1) / 2;
   
   for (i=0; i<skydim; i++) skyim[i] = 0.0;

   for (i=indx[0]; i<skydim-indx[0]; i++) {
      
      for (j=0; j<6; j++) {max[j] = -100000.0, max_j[j] = -1;}
      for (j=0; j<6; j++) {min[j] = 100000.0, min_j[j] = -1;}
      for (j=0; j<20; j++) {
         v = sky_r[i+indx[j]];
         if ((first == 1) && (logger->test == 1) && (i == sky_h)) {
            sprintf(logger->tmptxt, "  %3d   %8d/%8.2f     %8d/%8.2f  %8d/%8.2f   %8d/%8.2f\n", 
               j, i+indx[j], v, max_j[0], max[0], max_j[1], max[1], max_j[2], max[2]);
            logger->logstat = logprint(logger->tmptxt, logger->lognum); 
         }
               
         for (n=0; n<2; n++) {
            if (v > max[n]) {
               for (k=1; k>n; k--) {
                  max[k] = max[k-1];
                  max_j[k] = max_j[k-1];
               }
               max[n] = v;
               max_j[n] = i+indx[j];
               goto nextv1;
            }
         }
nextv1:  for (n=0; n<2; n++) {
            if (v < min[n]) {
               for (k=1; k>n; k--) {
                  min[k] = min[k-1];
                  min_j[k] = min_j[k-1];
               }
               min[n] = v;
               min_j[n] = i+indx[j];
               goto nextv2;
            }
         }
nextv2: ;
      }

      b = 0.0;
      n = 0;
      
      for (j=0; j<20; j++) {
         k = i+indx[j];
         logger->tmptxt[0] = 0;
         if ((first == 1) && (logger->test == 1) && (i == sky_h)) {
            sprintf(logger->tmptxt2, "%5d:  %7.2f ", indx[j], sky_r[k]);
            strcat(logger->tmptxt, logger->tmptxt2);
         }

         for (m=0; m<2; m++) {
            if (k == max_j[m]) goto try2;
            if (k == min_j[m]) goto try2;
         }
         if ((first == 1) && (logger->test == 1) && (i == sky_h)) {
            sprintf(logger->tmptxt2, "  %2d   %2d   %8d  %8.2f\n", j, n, i, b);
            strcat(logger->tmptxt, logger->tmptxt2);
         }
         b += sky_r[k];
         n++;
         goto next_j;
         
try2:    if ((first == 1) && (logger->test == 1) && (i == sky_h)) {
            sprintf(logger->tmptxt2, "* %2d   %2d   %8d  %8.2f\n", j, n, i, b);
            strcat(logger->tmptxt, logger->tmptxt2);
         }
next_j:
   	 if (strlen(logger->tmptxt) > 0) logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      }
      
      b /= (float)(n);
      
      skyim[i] = sky_r[i] - b;

      if ((first == 1) && (logger->test == 1) && (i == sky_h)) first = 0;
   }
   
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return;
}

/* ********************************************************** */

void clean2(float sky_r[skydim], float xx[], float yy[], int n_common, int backmodls, 
   float skyim[skydim], struct log_con *logger, struct backpro *backproj)
{
   int i, j, n, k, m, jj, ix, iy, ii;
   int indx[10] = { 5, sky_ydim-4, sky_ydim+4, 2*sky_ydim-3, 2*sky_ydim+3, 
      		3*sky_ydim-2, 3*sky_ydim-2, 4*sky_ydim-1, 4*sky_ydim+1, 5*sky_ydim};
   float v, sky_l[skydim], b, r_scale;
   double r[25], nr[25], r2[25], rc[25], nrc[25], rc2[25];
   double dx, dy, dj, dk, rr;
   
   static int func_local = -1, local_calling_num=0;
   clock_t calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "clean2");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
   r_scale = (float)(backproj->FOV_radius) / 25.0;
   for (i=0; i<skydim; i++) {
      skyim[i] = 0.0;
      sky_l[i] = sky_r[i];
   }
   for (i=0; i<25; i++) r[i] = nr[i] = r2[i] = rc[i] = nrc[i] = rc2[i] = 0.0;

   for (n=0; n<n_common; n++) {
      ix = (int)(xx[backmodls+n] + 0.5);
      iy = (int)(yy[backmodls+n] + 0.5);
      ii = iy + ix * sky_ydim;
      v = 0.0;
      m = 0;
      for (i=0; i<10; i++) {
         j = ii + indx[i];
         if (j < 0) continue;
         if (j >= skydim) goto conti;
         v += sky_r[j];
         m++;
conti:   j = ii - indx[i];
         if (j < 0) continue;
         if (j >= skydim) continue;
         v += sky_r[j];
         m++;
      }
      if (m > 0) v /= (float)(m); else continue;
      
      for (i=-2; i<3; i++) {
         for (j=-2; j<3; j++) {
            jj = ii + j * sky_ydim + i;
            sky_l[jj] = v;	/* replace peak by average level */
         }
      }
   }
   

   for (ii=5*sky_ydim; ii<skydim-5*sky_ydim; ii++) {

      v = 0.0;
      m = 0;
      for (i=0; i<10; i++) {
         j = ii + indx[i];
         if (j < 0) continue;
         if (j >= skydim) goto conti2;
         v += sky_l[j];
         m++;
conti2:  j = ii - indx[i];
         if (j < 0) continue;
         if (j >= skydim) continue;
         v += sky_l[j];
         m++;
      }
      if (m > 0) v /= (float)(m); else continue;
      
      skyim[ii] = sky_l[ii] - v;	/* remove local mean level */
   }
   
   dx = xx[backmodls] - (float)(backproj->FOV_radius);
   dy = yy[backmodls] - (float)(backproj->FOV_radius);

   for (i=0; i<skydim; i++) {
      j = i / sky_ydim;
      k = i % sky_ydim;
      dj = j - backproj->FOV_radius;
      dk = k - backproj->FOV_radius;
      rr = sqrt((dx - dj) * (dx - dj) + (dy - dk) * (dy - dk)) / 6.0;
      m = rr;
      rr = sqrt(dj * dj + dk * dk);
      if (m > 24) m = 24;
      if (rr > (double)(backproj->FOV_radius-1)) continue;
      r[(int)(rr / r_scale)] += skyim[i];
      r2[(int)(rr / r_scale)] += skyim[i] * skyim[i];
      nr[(int)(rr / r_scale)] += 1.0;
      rc[m] += skyim[i];
      rc2[m] += skyim[i] * skyim[i];
      nrc[m] += 1.0;
   }
   for (i=0; i<25; i++) {
      if (nr[i] > 0.0) r2[i] = sqrt(r2[i] * nr[i] - r[i] * r[i]) / nr[i];
      if (nrc[i] > 0.0) rc2[i] = sqrt(rc2[i] * nrc[i] - rc[i] * rc[i]) / nrc[i];
   }

   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return;
}

/* ********************************************************** */

float peaktransfer(float x, float y, float sky_r[skydim], float impeak[skydim], 
   float *sum2, struct backpro *backproj, struct log_con *logger)
{
   int ix, iy, ixy, jj;
   int i, j, n, k;
   float b, v, sum=0;
   
   static int func_local = -1, local_calling_num=0;
   clock_t calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "peaktransfer");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
   ix = x + 0.5;
   iy = y + 0.5;
   ixy = ix * sky_ydim + iy;	/* index for peak center */
   
   *sum2 = 0.0;
   for (i=-2; i<3; i++) {
      for (j=-2; j<3; j++) {
         jj = ixy + j * sky_ydim + i;
         impeak[jj] = sky_r[jj];	/* save peak counts into 'impeak'-image */
         sum += impeak[jj];
         *sum2 += impeak[jj] / (float)(backproj->illum[jj]);         
      }
   }
   
   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   return(sum);
}

/* ************************************************************************************* */

void fi_headr(char fitsfil[], struct log_con *logger)
{
   FILE *fi_fil;
   char fi_line[81];
   int i, j;
   
   static int func_local = -1, local_calling_num=0;
   clock_t calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "fi_headr");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
   fi_fil = fopen(fitsfil, "wt");
   
   sprintf(fi_line, "SIMPLE  =                    T / Written by mb9 version 3.0 August 23, 2004  ");    
   for (i=74; i<80; i++) fi_line[i] = 32;
   fi_line[80] = 0;
   fprintf(fi_fil, "%s", fi_line);
   sprintf(fi_line, "BITPIX  =                    8 /  ");    
   for (i=32; i<80; i++) fi_line[i] = 32;
   fi_line[80] = 0;
   fprintf(fi_fil, "%s", fi_line);
   sprintf(fi_line, "NAXIS   =                    0 /  ");    
   for (i=32; i<80; i++) fi_line[i] = 32;
   fi_line[80] = 0;
   fprintf(fi_fil, "%s", fi_line);
   sprintf(fi_line, "EXTEND  =                    T / File contains extension  ");    
   for (i=56; i<80; i++) fi_line[i] = 32;
   fi_line[80] = 0;
   fprintf(fi_fil, "%s", fi_line);
   sprintf(fi_line, "DATGROUP=              UNKNOWN /  ");    
   for (i=32; i<80; i++) fi_line[i] = 32;
   fi_line[80] = 0;
   fprintf(fi_fil, "%s", fi_line);
   sprintf(fi_line, "END  ");    
   for (i=3; i<80; i++) fi_line[i] = 32;
   fi_line[80] = 0;
   fprintf(fi_fil, "%s", fi_line);
   
   for (i=0;  i<80; i++) fi_line[i] = 32;
   fi_line[80] = 0;
   for (j=6; j<36; j++) fprintf(fi_fil, "%s", fi_line);

   /* function exit timing start */
   TT_now = clock();
   logger->func_times[func_local] += difftime(TT_now, logger->TT);
   logger->TT = TT_now;
   logger->func_calling_num = local_calling_num;
   /* function exit timing end */
   
   fclose(fi_fil);
}
/* ***************************************************************************************** */

int alignment_init(int jemx, int FOV_ff, struct log_con *logger, struct instr_data *jmx_id,
   	struct backpro *backproj, struct euclid *g_const)
{
      FILE *mhfil;
      char alfil[200], mhline[410], maskfil[200], *charpt;
      float indat[2], pixmm, mask_scale;
      int i, j, n;
      
   static int func_local = -1, local_calling_num=0;
   clock_t calling_TT, TT_now;

   /* function entry timing start */
   if (func_local < 0) {
      func_local = logger->func_used_num++;
      sprintf(logger->func_names[func_local], "alignment_init");
      if (logger->func_used_num > 199) {printf("func_used_num > 199\n"); exit(0);}
   }
   logger->func_calls[func_local]++;
   calling_TT = logger->TT;
   logger->TT = TT_now = clock();
   local_calling_num = logger->func_calling_num;
   logger->func_calling_num = func_local;
   logger->func_times[local_calling_num] += difftime(TT_now, calling_TT);
   /* function entry timing end */
   
      sprintf(alfil, "alignment8.dat");
      mhfil = fopen(alfil, "rt");
      if (mhfil != NULL) goto al_sta;

      sprintf(alfil, "%s/reference/alignment8.dat", logger->sweethome);
      mhfil = fopen(alfil, "rt");
      if (mhfil == NULL) {
         sprintf(logger->tmptxt, "No alignment file found! (%s)\n", alfil);
         logger->logstat = logprint(logger->tmptxt, logger->lognum); 
         /* function exit timing start */
         TT_now = clock();
         logger->func_times[func_local] += difftime(TT_now, logger->TT);
         logger->TT = TT_now;
         logger->func_calling_num = local_calling_num;
         /* function exit timing end */
          
         return(-1);
      } else {
al_sta: jmx_id->coll_wb = 0.0;
        jmx_id->coll_curv = 1.0; 
        backproj->signiflim = 100;
   	for (i=0; i<200; i++) {
            fgets(mhline, 200, mhfil);
            if (feof(mhfil)) goto q_end;
            sscanf(mhline, "%d", &n);
            if (n == 1) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->coll_pitch = indat[jemx];}
            if (n == 2) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->coll_h = indat[jemx];}
            if (n == 3) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->coll_w = indat[jemx];}
            if (n == 4) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->coll_r = indat[jemx];}
            if (n == 5) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->mask_r = indat[jemx];}
            if (n == 6) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->mask_height = indat[jemx];}
            if (n == 7) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->hexa_dim = indat[jemx];}

            if (n == 10) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->alfa = indat[jemx];}
            if (n == 11) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->beta = indat[jemx];}
            if (n == 12) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->gammav = indat[jemx];}
            if (n == 13) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->xshi = indat[jemx];}
            if (n == 14) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->yshi = indat[jemx];}
            if (n == 15) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->c_tiltx = indat[jemx];}
            if (n == 16) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->c_tilty = indat[jemx];}

            if (n == 17) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->coll_wb = indat[jemx];}

            if (n == 22) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->coll_curv = indat[jemx];}
            if (n == 23) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); mask_scale = indat[jemx];}
            
            if (n == 30) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); x_rms[0] = indat[jemx];}
            if (n == 31) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); x_rms[1] = indat[jemx];}
            if (n == 32) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); x_rms[2] = indat[jemx];}
            if (n == 33) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); x_rms[3] = indat[jemx];}
            if (n == 34) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); x_rms[4] = indat[jemx];}
            if (n == 35) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); rmsfact = indat[jemx];}

            if (n == 40) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); y_rms[0] = indat[jemx];}
            if (n == 41) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); y_rms[1] = indat[jemx];}
            if (n == 42) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); y_rms[2] = indat[jemx];}
            if (n == 43) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); y_rms[3] = indat[jemx];}
            if (n == 44) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); y_rms[4] = indat[jemx];}

            if (n == 100) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->scaleX = indat[jemx];}
            if (n == 101) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->scaleY = indat[jemx];}
            if (n == 102) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->phase2 = indat[jemx];}
            if (n == 103) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->x2x = indat[jemx];}
            if (n == 104) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->y2y = indat[jemx];}
            if (n == 105) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->offX = indat[jemx];}
            if (n == 106) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->offY = indat[jemx];}
            if (n == 107) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->offQ = indat[jemx];}
            if (n == 108) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); jmx_id->offR = indat[jemx];}

            if (n == 110) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); backproj->signiflim = indat[jemx];}
         }
      }
q_end:

      for (i=0; i<5; i++) {x_rms[i] *= rmsfact; y_rms[i] *= rmsfact;}

      jmx_id->hexa_dim *= mask_scale;
      jmx_id->mask_height *= mask_scale;

      jmx_id->offX *= FOV_ff;
      jmx_id->offY *= FOV_ff;
      backproj->FOV_radius = 128 * FOV_ff - 1;
      
      pixmm = (jmx_id->coll_r + jmx_id->mask_r) / backproj->FOV_radius;
      backproj->pix2rad = pixmm / jmx_id->mask_height;
      
      sprintf(logger->tmptxt, "\nRead parameters: %lf %lf %lf %lf %lf %lf\n", 
         jmx_id->coll_pitch, jmx_id->coll_h, jmx_id->coll_w, jmx_id->coll_r, 
         jmx_id->mask_r, jmx_id->hexa_dim);
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      sprintf(logger->tmptxt, "Read parameters: %lf %lf %lf %lf %lf %lf %f %f %lf %lf\n", 
         jmx_id->alfa, jmx_id->beta, jmx_id->gammav, jmx_id->xshi, jmx_id->yshi, 
         jmx_id->mask_height, jmx_id->coll_wb, rmsfact);
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
      sprintf(logger->tmptxt, "Read scale parameters: %lf %lf %lf %lf %lf %lf %lf %lf %lf, derived pix2rad: %10.8lf\n", 
         jmx_id->scaleX, jmx_id->scaleY, jmx_id->phase2, jmx_id->x2x, jmx_id->y2y, 
         jmx_id->offX, jmx_id->offY, jmx_id->offQ, jmx_id->offR, backproj->pix2rad);
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 

      fclose(mhfil);

      jmx_id->offQ *= backproj->pix2rad;
      jmx_id->offR *= backproj->pix2rad;

      jmx_id->coll_wb += jmx_id->coll_w;
      jmx_id->alfa *= g_const->gtorad;
      jmx_id->beta *= g_const->gtorad;
      jmx_id->gammav *= g_const->gtorad;

      sprintf(maskfil, "%s/reference/maskhole.txt", logger->sweethome);
/*      sprintf(logger->tmptxt, "Opening: %s", maskfil);
      logger->logstat = logprint(logger->tmptxt, logger->lognum); 
*/
      mhfil = fopen(maskfil, "rt");
      if (mhfil == NULL) {
         sprintf(logger->tmptxt, "Could not open maskhole.txt\n"); 
         logger->logstat = logprint(logger->tmptxt, logger->lognum); 
         /* function exit timing start */
         TT_now = clock();
         logger->func_times[func_local] += difftime(TT_now, logger->TT);
         logger->TT = TT_now;
         logger->func_calling_num = local_calling_num;
         /* function exit timing end */
          
         return(-1);
      }
      
      for (i=0; i<200; i++) {	/* read NJW definition of open mask holes */
         fgets(mhline, 410, mhfil);
         for (j=0; j<200; j+=20) {
            charpt = &mhline[j*2];
            sscanf(charpt, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
               &jmx_id->njwmask[i][j+0], &jmx_id->njwmask[i][j+1], 
               &jmx_id->njwmask[i][j+2], &jmx_id->njwmask[i][j+3], 
               &jmx_id->njwmask[i][j+4], &jmx_id->njwmask[i][j+5], 
               &jmx_id->njwmask[i][j+6], &jmx_id->njwmask[i][j+7], 
               &jmx_id->njwmask[i][j+8], &jmx_id->njwmask[i][j+9], 
               &jmx_id->njwmask[i][j+10], &jmx_id->njwmask[i][j+11], 
               &jmx_id->njwmask[i][j+12], &jmx_id->njwmask[i][j+13], 
               &jmx_id->njwmask[i][j+14], &jmx_id->njwmask[i][j+15], 
               &jmx_id->njwmask[i][j+16], &jmx_id->njwmask[i][j+17], 
               &jmx_id->njwmask[i][j+18], &jmx_id->njwmask[i][j+19]);
         }
      }
      fclose(mhfil);
      
      /* function exit timing start */
      TT_now = clock();
      logger->func_times[func_local] += difftime(TT_now, logger->TT);
      logger->TT = TT_now;
      logger->func_calling_num = local_calling_num;
      /* function exit timing end */
   
      return(1);
}

/* **************************************************************************************** */

int logprint(char *text, int lm)
{
   static q = 0;
   
   printf("0 %s", text);
   q++;
/*   if (q > 200) exit(0);
*/   
   
   return(0);
}

/* **************************************************************************************** */

