/*
Constants, global variables and function prototypes for midi_b
 *** In all of the midi_b routines the variable "jemx" has the value 0 for JEM-X1 and 1 for JEM-X2 
*/

/* #include "/soft/saord/include/funtools.h" */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/timeb.h>
#include <funtools.h>

#define FOV_f		1		/* FOV_f=1 means 255x255 skymaps, FOV_f=2 means 511x511 skymaps */
#define FOV_rad         (128*FOV_f-1)   /* field of view radius in pixels */
#define sky_xdim        (2*FOV_rad+1)  	/* x-dimension of skymap */
#define sky_ydim        (2*FOV_rad+1)  	/* y-dimension of skymap */
#define skydim          65025	/* sky_xdim*sky_ydim */ 	/* length of (linear) skymap array */

#define max_sky_bytes	(200000000*FOV_f)	/* max number of bytes in d_allow array */
#define max_candidates	20		/* max number of sky_map peak candidates accepted in search */ 
#define detdim 		65536		/* lenght of linearized detector array (256X*256Y) */
#define det_ydim	256

#define sourcmax	20		/* max number of components in shadowgram fits */
#define accumax		10000		/* max number of multiple detection source candidates */

#define SWAP(a,b) {double temp=(a);(a)=(b);(b)=temp;}	/* function used in "gaussj_nr" */

struct euclid {
	double pi;
	double twopi;
	double gtorad;
};

struct instr_data {
    double coll_pitch;
    double coll_h;       /* height of collimator */
    double coll_w;       /* halfthickness of lamellae */
    double coll_r;	/* collimator radius */ 
    double xshi;
    double yshi;
    double c_tiltx;
    double c_tilty;
    float  coll_wb;
    double coll_curv;

    double mask_r;	/* radius of coded mask pattern */
    double mask_height;
    double hexa_dim;     /* flat-to-flat dimension of the mask elements */
    int njwmask[200][200]; 

    double alfa;
    double beta;
    double gammav;
/* +++++++++++++++++++ JEM-X to sky conversion */
    double phase2;
    double x2x;
    double y2y;
    double offX;
    double offY;
    double scaleX;
    double scaleY;
    double offQ;
    double offR;
    double pix2rad;
/* ++++++++++++++++++++ absorbtion & position uncertainty */
    double absolen;
    double scatt_len;
    int    scatt_delta[500];
    float  scatt_list[500];

    int    pixel_st[detdim];
    unsigned char pixel_list[500000];
};

struct log_con {	/* logging text strings and control parameters */
   char tmptxt[200];
   char tmptxt2[128]; 
   char error_text[128];
   int  logstat; 
   int  lognum;
   
   int  test;
   int  o_ctrl;

   char func_names[200][25];
   int  func_used_num;
   int  func_calling_num;
   double func_times[200];
   int  func_calls[200];
   clock_t TT;

   char sweethome[200];
   char sweethome2[200];

};

struct backpro {
    int    FOV_radius;
    double indep; 
    double depth_corr;
    int    backmodels; 
    double pix2rad;

    float  soft_gr[detdim];
    float  empty[4][detdim];
    float  shadw[detdim];

    double empty_aux[30];
    double soft_tot[30];
    double parameters[20];

    float  rsignif[skydim];
    float  fitsimage[4][5][skydim];
    float  skyim_r[skydim];
    char   access[skydim];
    int    illum[skydim];

    float  com_RA[20];
    float  com_dec[20];
    
    int    signiflim;
    int    aux_orbit;
    int    aux_pid; 
    int    aux_pidv;
};

struct imagerec {
   double icen[2];
   int    idim[2];
   double idel[2];
   double irot[2][2];
   char   imagtype[255];
   char   imagextn[255];
   double ee_limits[2];
   char   instrum[10];
   double rotang;
   double crpix[2];
   double ontime;
   int    jmx_unit;
   int    revol;
   int    pid;
   int    pidv;
   int    n_extract;
   char   comment[5][128];
};



int    backpro_d(float shdgr[detdim], float skyim[skydim], struct log_con *logger, struct backpro *bacproj);
int    backpro_init(char skyfil[], double parameters[], struct log_con *logger);
   
int    findpeak(float *skyimage, int *smaxx, int *smaxy, double *amax, float edge,
   		struct log_con *logger, struct backpro *backproj);
float  fturn(int jemx, float pos, struct log_con *logger);
int    imagefit(float shdg[], int jemx, int ee, float resid[], int n_source, float srcx[], float srcy[], 
          	double *solut, float norm_d[], float sumfun[], int last, double *rms, struct euclid *g_const, 
                struct instr_data *jmx_id, struct log_con *logger, struct backpro *backproj);
int    fun(int k, int ii, int ee, float srcx[], float srcy[], int jemx, float norm_d[], int last, 
   	  	float *value, struct euclid *g_const, struct instr_data *jmx_id, 
                struct log_con *logger, struct backpro *backproj);
int    fun2(int k, int ii, float *value, struct log_con *logger);
int    gaussj_nr(double *a, int n, double *b, int m, struct log_con *logger);

int    detecmap(float x_source, float y_source, char srcfun[detdim], int last, int jemx, 
   		float *norm, struct euclid *g_const, struct instr_data *jmx_id, 
                struct log_con *logger, struct backpro *backproj);

int    pixelfold(short det[], int ee, float shadow[detdim], struct log_con *logger, struct backpro *backproj,
   		struct instr_data *jmx_id);
double rmscalc(float shad[], float soft_gr[], struct log_con *logger);
double rmscalc_i(short sh[], float soft_gr[], struct log_con *logger);
double shapefold(char det[], float comb_list[], int comb_delta[], int comb_len, float det_out[], 
   		int last, struct log_con *logger);
int    shapecomb(float track_list[], int track_delta[], int track_len, float comb_list[], int comb_delta[], 
                struct log_con *logger, struct instr_data *jmx_id);
int    irosloop(int jemx, int ee, float xx[sourcmax], float yy[sourcmax], float norm_d[sourcmax], 
                double solut[sourcmax], float sumfun[sourcmax], float *goodrms, double *sh_rms2, int *goodfits, 
                int *goodnsrc, float resid[detdim], float edge, float goodbet_g[sourcmax], int iterations, 
                double limit_fraction, struct euclid *g_const, struct instr_data *jmx_id, struct log_con *logger, 
                struct backpro *backproj);
int    getRAdec_init(int jemx, double p_RA[10000], double p_dec[10000], double p_posang[10000], 
                 struct euclid *g_const, struct log_con *logger, struct instr_data *jmx_id, struct backpro *backproj);
int    getRAdec(float x, float y, int jemx, float *s_RA, float *s_dec, 
   		struct euclid *g_const, struct log_con *logger, struct instr_data *jmx_id, struct backpro *backproj);
int    getxy(float s_RA, float s_dec, int jemx, float *x, float *y, struct euclid *g_const, 
   		struct instr_data *jmx_id, struct backpro *backproj, struct log_con *logger);
void   clean(float skyim_r[skydim], float skyim[skydim], struct log_con *logger);
void   clean2(float sky_r[skydim], float xx[], float yy[], int n_common, int backmodels,  
   		float skyim[skydim], struct log_con *logger, struct backpro *backproj);
int    logprint(char *text, int lognum);
int    struchk(double xm, double ym, double x_det, double y_det, int last, struct euclid *g_const,
   		struct instr_data *jmx_id, struct log_con *logger);
int    sh_xy(double x_rms, double y_rms, struct log_con *logger, struct instr_data *jmx_id);
int    sh_depth(int sky_index, float track_list[], int track_delta[], int jemx, 
   			struct instr_data *jmx_id, struct log_con *logger, struct backpro *backproj);
int    sign(int x, struct log_con *logger);


